Attribute VB_Name = "Main"
Option Explicit
'(C)Roy E. Hoffman 2000
'Permission is given to use this software for non-commercial
'religious and scientific purposes. If you wish to use this
'software for commercial purposes, please contact me to discuss
'terms. If you have any comments
'or suggestions, please contact me (roy@shum.cc.huji.ac.il).
'While I will do my best to acknowledge any suggestions in
'the code, I will assume that I have your permission to use
'any information you give me as I wish.

'Many of the algorithms used here are based on the books
'"Calendrical Calculations" by Reingold and
'"Astronomical Algorithms" by Meeus

'V1.0 6/2/00
'V1.1 21/2/00 Added defaults in all fields
'C2.0 16/4/00 Redesigned data formats
'Corrected problems with comversion of negative DMS format
Public Const RADCON = 57.2957795130823 'Degrees in a radian
Public Const TwoPi = 6.28318530717959
Public Const SQRT2 = 1.4142135623731
Public Const TMin = -100 'Minimum temperature for weather in C
Public Const TMax = 80   'Maximum temperature for weather in C
Public Const PMin = 100  'Minimum pressure for weather in mbar
Public Const PMax = 1150 'Maximum pressure for weather in mbar
Public Const HMin = 0    'Minimum relative humidity for weather in %
Public Const HMax = 100  'Minimum relative humidity for weather in %
Public Const HeightMin = -1000  'Minimum height above sea level in m
Public Const HeightMax = 100000 'Maximum height above sea level in m
'Weather types
Public Type Range 'Used to estimate the weather
    dblMean As Double 'Average
    dblAnnualRange As Double 'Max - min on anuual basis
    dblDailyRange As Double 'On daily basis
End Type
Public Type Climate 'Weather parameters for calculating at any instant
    udtTemperature As Range
    udtPressure As Range
    udtHumidity As Range
End Type
Public Type WeatherNow 'Weather at a particular instant
    dblTemperature As Double
    dblPressure As Double
    dblHumidity As Double
End Type
Public Enum WeatherBasis 'How the weather is caluclated
    GlobalW 'Based on global parameters
    LocalW 'Based on the nearest local parameters
    Manual 'Manually entered
End Enum
Public Type Weather 'Weather parameters for a location
    udtBasis As WeatherBasis
    udtClimate As Climate
End Type
Public Enum DateType 'Type of date
    'Radix dates that use fractional days instead of times
    Fixed 'Fixed radix relative to 1/1/1 Gregorian = 1
    JD  'Julian day number in UTC
    JDE 'Julian day number in dynamic time
    mjd 'Modified Julain day number in UTC
    'Calendar dates that require a time component
    Gregorian
    Julian
    Hebrew
    Islamic 'Arithmetic Islamic
End Enum
Public Enum TimeType 'Type of time
    utc 'Universal Coordinated Time
    localt 'Local time according to convention
    meansolar 'Natural local time according to the average position of the Sun
    truesolar 'Time according to a Sundial, i.e. the true position of the Sun (not correcting for refraction)
    gmst 'Greenwich sidreal time
    lst 'Local sidreal time
    seasonal 'Time according to seasonal hours, counting Sunset as 0
End Enum
Public Type MyDate 'Date structure
    enmDateType As DateType
    dblDayNumber As Double 'Number for radix dates
    intDateOfMonth As Integer
    intMonth As Integer
    intYear As Integer 'Year may be negative except for Julian
    blnCE As Boolean 'For Julian date
    blnEve As Boolean
End Type
Public Type DMS 'Base 60 system
    intDegreesHours As Integer
    intMinutes As Integer
    dblSeconds As Double
    blnSign As Boolean 'TRUE = positive
End Type
Public Type Mytime 'Time structure
    enmTimeType As TimeType
    bln24Hr As Boolean '24 hour or am/pm mode
    blnPM As Boolean 'for seasonal time this means day if true
    udtDms As DMS 'The time in hours/minutes/seconds
    'Sometimes, a time may occur twice in on day, e.g.
    '1) sidreal time repeats after about 23h 56m so about 4m repeats itself
    '2) in the Hebrew and Islamic calendars, the day starts at sunset so if
    'the the sun sets later the next day then the time can occur twice
    'Hence there is a need to allow for time to repeat itself
    blnSecondTime As Boolean
End Type
Public Type ClockChange 'Defines date and time of change to and from DST
    intHour As Integer 'Hour at wich the clock is changed from
    intDayOfWeek As Integer 'Day of the week is ignored for intWeekNo = 0
    intWeekNo As Integer 'Week after date to follow, 0 = on the date itself, 5 = last week in month
    intDate As Integer 'Date in month from which the weeks are counted
    intMonthNo As Integer
End Type
Public Type TimeZone 'Defines the automatic time zone parameters for local time
    blnLast As Boolean 'Was there a last applicable date
    dblLast As Double 'Last applicable date in fixed radix
    dblTZ As Double 'Hours ahead of UTC
    blnDST As Boolean 'Is it DST for manual setting or where auto is ambiguous
    udtDSTStart As ClockChange 'Start of DST in spring
    udtDSTEnd As ClockChange 'End of DST in fall
End Type
'Location types
Public Type Location 'Parameters defining location
    blnDMSMode As Boolean 'Parameters in Deg or Hr/Min/Sec or decimal degrees/hours
    blnNameValid As Boolean 'True if the name of the location matches the coordinates
    strCountryState As String 'Country (State/Province in USA, Canada and Australia)
    strTownCity As String 'Town, city or, if not in a populated area, location
    strNeighborhood As String 'Neighborhood or "Center" if not defined
    'Positional coordinates
    dblLatitude As Double
    udtLatitude As DMS 'N/S is udtLatitude.blnSign
    dblLongitude As Double
    udtLongitude As DMS 'E/W is udtLongitude.blnSign
    dblHeight As Double 'Height in m above sea-level
    'Time zone
    dblTZ As Double 'Time zone for manual setting or when auto is ambiguous
    intTZH As Integer 'Manual time zone data
    intTZM As Integer
    blnAhead As Boolean
    blnDST As Boolean 'Is it DST for manual setting or where auto is ambiguous
    blnDSTAuto As Boolean 'Set DST automatically where possible
    intNTZ As Integer 'Number of time zone settings throughout history
    udtTZ() As TimeZone 'Automatic time zone parameters
End Type
'Structure of the locations database
Public Type Position
    dblLatitude As Double
    dblLongitude As Double
    dblHeight As Double
End Type
Public Type Neighborhood
    strNeighborhood As String
    udtPosition As Position
    udtLocalClimate As Climate 'Weather data for location
End Type
Public Type City
    strCity As String
    intNNeighborhoods As Integer
    udtNeighborhood() As Neighborhood
End Type
Public Type CountryState
    strCountryState As String
    intNTZ As Integer 'Number of time zone settings throughout history
    udtTZ() As TimeZone 'Auto time zone parameters
    intNCities As Integer
    udtCity() As City
End Type
Public Type LocationData
    intNStatesCountries As Integer
    udtCountryState() As CountryState
End Type
Public Type Vector
    N As Integer
    A() As Double
    B() As Double
    C() As Double
End Type
Public Type VSOP
    NL As Integer
    NB As Integer
    NR As Integer
    L() As Vector
    B() As Vector
    R() As Vector
End Type
Public Type HBody 'Heavenly body
    EclLong As Double 'Ecliptic longitude
    EclDecl As Double 'Ecliptic declination
    DistanceGeo As Double 'Geocentric distance
    DistanceTopo As Double 'Topocentric Distance
    RAGeo As Double 'Geocentric Right Ascention in degrees
    RATopo As Double 'Topocentric Right Ascention in degrees
    DeclGeo As Double 'Geocentric delcination (equatorial coordinates)
    DeclTopo As Double 'Geocentric delcination (equatorial coordinates)
    HourAngGeo As Double 'Geocentric Hour Angle in degrees
    HourAngTopo As Double 'Topocentric Hour Angle in degrees
    AltGeo As Double 'Geocentric Altitude
    AzGeo As Double 'Geocentric Azimuth
    AltTopo As Double 'Topocentric Altitude
    AzTopo As Double 'Topocentric Azimuth
    AltRefr As Double 'Topocentric Altitude corrected for refraction
    Radius As Double 'Angular radius of object (topocentric)
    Phase As Double 'Phase (topocentric)
    ElongationGeo As Double 'Angular separation from the Sun
    ElongationTopo As Double 'Angular separation from the Sun
End Type
'Public variables
Public strDecimalSymbol As String
Public udtMercury As VSOP
Public blnMercury As Boolean
Public udtVenus As VSOP
Public blnVenus As Boolean
Public udtEarth As VSOP
Public blnEarth As Boolean
Public udtJupiter As VSOP
Public blnJupiter As Boolean
'Input values
Public udtTime As Mytime 'Time
Public udtDate As MyDate 'and date entered
Public dblDate As Double 'Fixed radix days at 0:00 UTC if from non-radix date
Public dblTime As Double 'Time in days from dblDate
Public udtLocation As Location 'Location parameters
Public udtWeather As Weather 'Weather parameters
'List of locations
Public udtLocationData As LocationData
Public udtPosition As Position 'Current position as long/lat/height
Public udtWeatherNow As WeatherNow 'Current weather
Public dblDeltaT(191) As Double 'List of DeltaT values for 1620-2000
Public blnDeltaTRead As Boolean 'Flag to show if read
Public lngMoonLR(5, 59) As Long 'Moon longitude and distance parameters
Public lngMoonB(4, 59) As Long 'Moon declination parameters
Public blnMoonRead As Boolean 'Flag for Moon parameters read
Public enmLastDateType As DateType 'Used to backtrack on type change
Public enmLastTimeType As TimeType 'Used to backtrack on type change
Public intLastPar As Integer 'Used to store last Sun output option
Public intLastPlanet As Integer
'Math functions
Public Function MyMod(lngA, lngB) As Long 'Produces a positive result for negative lngA
    MyMod = lngA Mod lngB
    If MyMod < 0 Then MyMod = lngB + MyMod
End Function
Public Function Frac(dblX As Double) As Double 'Produces a fraction
    Frac = dblX - Int(dblX)
End Function
Public Function dblMod(dblX As Double, intI As Integer) As Double 'Mod that returns a positive number
    dblMod = Frac(dblX / intI) * intI
End Function
Public Function FracCirc(dblX) As Double 'Produces a value 0 <= FracCirc < 360
    FracCirc = dblX - 360 * Int(dblX / 360)
End Function
Public Function FracRadCirc(dblX) As Double 'Produces a value 0 <= FracCirc < 360
    FracRadCirc = dblX - TwoPi * Int(dblX / TwoPi)
End Function
Public Function sinD(dblX As Double) As Double 'Sin of angle in degrees
    sinD = Sin(FracCirc(dblX) / RADCON)
End Function
Public Function cosD(dblX As Double) As Double 'Cos of angle in degrees
    cosD = Cos(FracCirc(dblX) / RADCON)
End Function
Public Function tanD(dblX As Double) As Double 'Tan of angle in degrees
    tanD = Tan(FracCirc(dblX) / RADCON)
End Function
Public Function acosD(dblX As Double) As Double 'Arcos to angle in degrees
    If dblX = 1 Then
        acosD = 0
    ElseIf dblX = -1 Then
        acosD = 180
    Else
        acosD = Atn(-dblX / Sqr(1 - Square(dblX))) * RADCON + 90
    End If
End Function
Public Function asinD(dblX As Double) As Double 'Arcsin to angle in degrees
    asinD = Atn(dblX / Sqr(1 - Square(dblX))) * RADCON
End Function
Public Function atn2D(dblY As Double, dblX As Double) As Double 'Arctan of dblY/dblX to angle in degrees in the correct quadrant
    If dblY = 0 Then
        If dblX >= 0 Then
            atn2D = 90
        Else
            atn2D = 270
        End If
    ElseIf dblX < 0 Then
        If dblY < 0 Then
            atn2D = Atn(dblY / dblX) * RADCON + 180
        Else
            atn2D = Atn(dblY / dblX) * RADCON + 180
        End If
    ElseIf dblY < 0 Then
        atn2D = Atn(dblY / dblX) * RADCON + 360
    Else
        atn2D = Atn(dblY / dblX) * RADCON
    End If
End Function
Public Function Square(dblX As Double) As Double 'dblX^2
    Square = dblX * dblX
End Function
Public Function Interp(dblY() As Double, dblX As Double, intN As Integer) As Double 'Do a 2, 3 or 5 point interpolation
'Suitable for eaqually spaced measurements
    Dim dblX2 As Double
    
    Select Case intN
        Case 2
            If dblX < 0 Or dblX > 1 Then
'                MsgBox ("X not in ideal range: " & dblX)
            End If
            Interp = (dblY(1) - dblY(0)) * dblX + dblY(0)
        Case 3
            If dblX < 0.5 Or dblX > 1.5 Then
'                MsgBox ("X not in ideal range: " & dblX)
            End If
            Interp = dblY(1) + (dblX - 1) / 2 * (dblY(2) - dblY(0) + (dblX - 1) * (dblY(2) - 2 * dblY(1) + dblY(0)))
        Case 5
            If dblX < 1.5 Or dblX > 2.5 Then
'                MsgBox ("X not in ideal range: " & dblX)
            End If
            dblX2 = dblX - 2
            Interp = dblY(2) + dblX2 / 2 * (dblY(3) - dblY(1)) + dblX2 * dblX2 / 2 * (dblY(3) - 2 * dblY(2) + dblY(1)) + dblX2 * (dblX2 * dblX2 - 1) / 12 * (dblY(4) - 2 * dblY(3) + 2 * dblY(1) - dblY(0)) * dblX2 * dblX2 * (dblX2 * dblX2 - 1) / 24 * (dblY(4) - 4 * dblY(3) + 6 * dblY(2) - 4 * dblY(1) + dblY(0))
        Case Else
            MsgBox ("Incorrect number of points in Interp: " & intN)
    End Select
End Function
Public Function Prob(dblX As Double) As Double
'Returns Normal Distribution function N(x) as defined by Feller (1971)
'Feller, W. An Introduction to Probability Theory and Its Applications, Vol. 2, 3rd ed. New York: Wiley, p. 45, 1971.
'N(x) = {1 + erf(x/sqr(2)]}/2
'erf(x) = 2/sqr(PI) integral(0-x) exp(-t^2)dt
    Dim dblT As Double
    Dim dblZ As Double
    
    dblZ = Abs(dblX / SQRT2)
    dblT = 1 / (1 + dblZ / 2)
    Prob = dblT * (1.48851587 + dblT * (-0.82215223 + dblT * 0.17087277))
    Prob = dblT * (-0.18628806 + dblT * (0.27886807 + dblT * (-1.13520398 + Prob)))
    Prob = -Square(dblZ) - 1.26551223 + dblT * (1.00002368 + dblT * (0.37409196 + dblT * (0.09678418 + Prob)))
    If Prob < -80 Then
        Prob = 0
    Else
        Prob = dblT * Exp(Prob)
    End If
    If dblX >= 0 Then
        Prob = 1 - Prob / 2
    Else
        Prob = Prob / 2
    End If
End Function
Public Function Log10(dblX As Double) As Double
    Log10 = Log(dblX) / 2.302585092994
End Function
Public Function ToIntNum(strLine As String) As String
    Dim intI As Integer
    
    ToIntNum = strLine
    For intI = 1 To Len(strLine) - 1
        If StrComp(Mid(strLine, intI, 1), ".") = 0 Then ToIntNum = Mid(strLine, 1, intI - 1) & strDecimalSymbol & Mid(strLine, intI + 1, Len(strLine) - intI - 1)
    Next
End Function
