VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsDeltaT"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit
'(C)Roy E. Hoffman 2000
'See Main.bas for terms and conditions of use


Function DeltaT(dblFixed As Double, Optional blnFromUTC As Boolean = True) As Double
'Find the difference between UTC1 and TD
'J. Meeus, "Astronomical Algorithms", 2nd Ed., William-Bell Inc., Richmond, VG, 1998, p. 77
'according to
'J. Chapront, M. Chapront-Touze and G. Francou, Note s055, bureau des Longitudes, Paris, Dec. 1997
'dblfixed is fixed radix from 1 jan 1 0:00 UTC gregorian = 1
    Dim dblJYear As Double
    Dim dblB1 As Double
    Dim dblB2 As Double
    Dim strLine As String
    Dim intI As Integer
    Dim dblY(3) As Double
    Dim dblT As Double
    Dim blnFinished As Boolean
    Dim dblOldDeltaT As Double
    Dim intN As Double
    
    If Not blnDeltaTRead Then 'First time, read table (biannual 1620 to 2000)
        Open "DeltaT.txt" For Input Access Read As 1
        For intI = 0 To 190
            If EOF(1) Then
                MsgBox ("Truncated deltat file.")
                Return
            End If
            Input #1, strLine
            strLine = ToIntNum(strLine)
            If Not IsNumeric(strLine) Then
                MsgBox ("Non numeric entry in deltat file.")
                Return
            End If
            dblDeltaT(intI) = strLine
        Next
        Close 1
        blnDeltaTRead = True
    End If
    DeltaT = 0
    dblOldDeltaT = 0
    blnFinished = False
    While Not blnFinished
        dblJYear = (dblFixed + 14.25 - DeltaT / 86400 + 365.25) / 365.25 'Julian year
        dblT = (dblJYear - 2000) / 100 'Julian centuries from 2000
        If dblJYear < 948 Then 'Distant past
            DeltaT = 2177 + dblT * (497 + dblT * 44.1)
        ElseIf dblJYear < 1600 Or dblJYear > 2000 Then 'Future and near past
            DeltaT = 102 + dblT * (102 + dblT * 25.3)
            If dblJYear < 2100 And dblJYear >= 2000 Then DeltaT = DeltaT + 0.382 * (dblJYear - 2100) 'To maintain continuity
        ElseIf dblJYear < 1620 Then 'No data between 1600 and 1620 so linear interpolation
            DeltaT = (dblJYear - 1600) * 1.11 + 98.8
        ElseIf dblJYear < 1621 Then 'Three point interpolation from biannual data (1621 to 1999)
            dblY(0) = dblDeltaT(0)
            dblY(1) = dblDeltaT(1)
            DeltaT = Interp(dblY, (dblJYear - 1620) / 2, 2)
        ElseIf dblJYear > 1999 Then '1999 to 2000 linear interpolation
            dblY(0) = dblDeltaT(189)
            dblY(1) = dblDeltaT(190)
            DeltaT = Interp(dblY, (dblJYear - 1998) / 2, 2)
        Else '1620 to 1621 linear interpolation
            dblY(0) = dblDeltaT((dblJYear - 1621) / 2)
            dblY(1) = dblDeltaT((dblJYear - 1621) / 2 + 1)
            dblY(2) = dblDeltaT((dblJYear - 1621) / 2 + 2)
            DeltaT = Interp(dblY, dblJYear / 2 - Int((dblJYear - 1) / 2), 3)
        End If
        blnFinished = blnFromUTC Or Abs(DeltaT - dblOldDeltaT) < 100 'if date in ephemeris time and year very large an iteration is required to maintain consistency
        dblOldDeltaT = DeltaT
    Wend
End Function
