VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsGMST"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit
'(C)Roy E. Hoffman 2000
'See Main.bas for terms and conditions of use

'Some of the algorithms from
'J. Meeus, "Astronomical Algorithms", 2nd Ed., William-Bell Inc., Richmond, VG, 1998, p. 87
Friend Function FromGMST(udtTime As Mytime, blnValid As Boolean) As Double
'Convert from GMST to a fraction of a day
    Dim cDMS As New clsDMS
    Dim udtLoc As Location
    Dim dblT As Double
    Dim dblT0 As Double
    Dim cHebrew As New clsHebrew
    
    blnValid = True
    'Julian centuries from 2000
    dblT = (Int(dblDate + 1721424.5 - 2451545)) / 36525 'Julian centuries at 0 UTC
    dblT0 = dblT0 - Int(dblT0)
    FromGMST = cDMS.FromDMS(udtTime.udtDms) / 24 - dblT0
    If udtTime.blnSecondTime Then FromGMST = FromGMST + 1
    FromGMST = (FromGMST - Int(FromGMST)) * 0.99726956633
    If udtTime.blnSecondTime Then FromGMST = FromGMST + 0.99726956633
    If FromGMST > 1 And udtDate.enmDateType < Hebrew Then
        blnValid = False
    ElseIf udtDate.enmDateType >= Hebrew Then
        Call cHebrew.AdjustTime(Int(dblDate), FromGMST, blnValid)
    End If
End Function
Friend Sub ToGMST(dblFix As Double, udtTimeIn As Mytime)
'convert a fixed radix to GMST
    Dim cDMS As New clsDMS
    Dim cSun As New clsSun
    Dim cUTC As New clsUTC
    Dim udtLoc As Location
    Dim dblT As Double
    Dim dblT0 As Double
    Dim dblTime2 As Double
    Dim dblStart As Double
    Dim dblEnd As Double
    Dim intDayShift As Integer
    
    udtTimeIn.bln24Hr = True
    udtTimeIn.enmTimeType = gmst
    dblT = (Int(dblFix) + 1721424.5 - 2451545) / 36525 'Julian date
    dblT0 = (6.697374558 + dblT * (2400.0513385167 + dblT * (0.000025862 - dblT * 0.00000000172))) / 24
    Call cUTC.StartEndOfDay(dblFix, udtDate.enmDateType, dblStart, dblEnd, intDayShift)
    dblT0 = dblT0 - Int(dblT0)
    udtTime.blnSecondTime = (dblFix - dblStart) / 0.99726956633 >= 1
    dblTime2 = (dblFix - Int(dblFix)) / 0.99726956633 + dblT0
    dblTime2 = dblTime2 - Int(dblTime2)
    Call cDMS.ToDMS(dblTime2 * 24, udtTimeIn.udtDms)
End Sub
'Friend Function ValidGMST(udtTime As Mytime, udtDate As MyDate) As Boolean
'    Dim udtTimeCheck As Mytime
'    Dim dblNoddle As Double
'
'    dblNoddle = 0.001 / 7200
'    Call ToGMST(FromGMST(udtTime, udtDate) + dblNoddle, udtTimeCheck, udtDate)
'    ValidGMST = (udtTime.udtDMS.blnSign And udtTime.udtDMS.blnSign = udtTimeCheck.udtDMS.blnSign And Abs(udtTime.udtDMS.dblSeconds - udtTimeCheck.udtDMS.dblSeconds) < 0.001 And udtTime.udtDMS.intDegreesHours = udtTimeCheck.udtDMS.intDegreesHours And udtTime.udtDMS.intMinutes = udtTime.udtDMS.intMinutes And udtTime.blnSecondTime = udtTimeCheck.blnSecondTime) ' And (Not udtTime.blnSecondTime Or FromGMST(udtTime, udtDate) < 1))
'End Function
'Friend Function StartOfDay(dblDate As Double, udtTime As Mytime)
'    StartOfDay = 0
'End Function
Friend Function FixToGMST(dblFix As Double, dblTD As Boolean) As Double 'In degrees
'dblFix is fixed radix in UTC or TD to GMST in degrees
    Dim dblT As Double
    Dim dblDateUTC As Double
    Dim cDeltaT As New clsDeltaT

    If dblTD Then
        FixToGMST = dblFix - cDeltaT.DeltaT(Int(dblFix), False)
    Else
        FixToGMST = dblFix
    End If
    dblT = (FixToGMST - 730120.5) / 36525
    FixToGMST = FracCirc(280.46061837 + 360.98564736629 * dblT * 36525 + dblT * dblT * (0.000387933 - dblT / 38710000))
End Function

