VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsMoon"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit
'(C)Roy E. Hoffman 2000
'See Main.bas for terms and conditions of use
Const AU = 149597870 'Astronomical unit in km

Friend Function MeanMoon(dblFix As Double) As Double
    'Mean Lunar ecliptic longitude given a fixed radix date
    Dim dblT As Double
    Dim cSun As New clsSun
    
    dblT = cSun.ToJ2000(dblFix)
'J. Meeus, "Astronomical Algorithms", 2nd Ed., William-Bell Inc., Richmond, VG, 1998, p. 338
        
    MeanMoon = dblT * (1 / 538841 - dblT / 65194000)
    MeanMoon = FracCirc(218.3164477 + dblT * (481267.88123421 - dblT * (0.0015786 - MeanMoon)))
End Function
Friend Sub TrueMoon(dblFix As Double, udtMoon As HBody)
    'Gets parameters for Moon at a fixed radix date
'Some of the algorithms from
'J. Meeus, "Astronomical Algorithms", 2nd Ed., William-Bell Inc., Richmond, VG, 1998, p. 338
'using ELP 2000-82B according to
'J. Chapront, M. Chapront-Touze and G.Francou, "Introduction dans ELP 2000-82B de nouvelles
'des parameters orbitaux de la Lune et du barycentre Terre-Lune", Paris, Jan. 1998.
    Dim dblLp As Double 'Moon's mean ecliptic longitude
    Dim dblD As Double  'Moon's mean ecliptic elongation
    Dim dblM As Double  'Sun's mean anomoly
    Dim dblMp As Double 'Moon's mean anomoly
    Dim dblF As Double  'Moon's argument of latitude
    Dim dblA1 As Double 'Venus
    Dim dblA2 As Double 'Jupiter
    Dim dblA3 As Double
    Dim dblE As Double  'Earth's eccentricity
    Dim cSun As New clsSun
    Dim intI As Integer
    Dim intJ As Integer
    Dim dblPar As Double
    Dim strLine As String
    Dim dblT As Double
    Dim dblOmega As Double
    Dim dblEE(2) As Double
    Dim udtSun As HBody
    
    
    If Not blnMoonRead Then 'First time, read table (biannual 1620 to 2000)
        Open "MoonLong.txt" For Input Access Read As 1
        Input #1, strLine 'First line is a comment
        For intI = 0 To 59
            For intJ = 0 To 5
                If EOF(1) Then
                    MsgBox ("Truncated MoonLR file.")
                    Return
                End If
                Input #1, strLine
                If Not IsNumeric(strLine) Then
                    MsgBox ("Non numeric entry in MoonLR file.")
                    Return
                End If
                lngMoonLR(intJ, intI) = strLine
            Next
        Next
        Close 1
        Open "Moonlat.txt" For Input Access Read As 1
        Input #1, strLine 'First line is a comment
        For intI = 0 To 59
            For intJ = 0 To 4
                If EOF(1) Then
                    MsgBox ("Truncated MoonB file.")
                    Return
                End If
                Input #1, strLine
                If Not IsNumeric(strLine) Then
                    MsgBox ("Non numeric entry in MoonB file.")
                    Return
                End If
                lngMoonB(intJ, intI) = strLine
            Next
        Next
        Close 1
        blnMoonRead = True
    End If
    dblT = cSun.ToJ2000(dblFix)
    dblLp = MeanMoon(dblFix)
    dblD = dblT * (1 / 545868 - dblT / 113065000)
    dblD = FracCirc(297.8501921 + dblT * (445267.1114034 - dblT * (0.0018819 - dblD)))
    dblM = FracCirc(357.5291092 + dblT * (35999.0502909 - dblT * (0.0001536 - dblT / 24490000)))
    dblMp = dblT * (1 / 69699 - dblT / 14712000)
    dblMp = FracCirc(134.9633964 + dblT * (477198.8675055 + dblT * (0.0087414 + dblMp)))
    dblF = dblT * (1 / 3526000 - dblT / 863310000)
    dblF = FracCirc(93.272095 + dblT * (483202.0175233 - dblT * (0.0036539 + dblF)))
    dblA1 = FracCirc(119.75 + dblT * 131.849)
    dblA2 = FracCirc(53.09 + dblT * 479264.29)
    dblA3 = FracCirc(313.45 + dblT * 481266.484)
    dblE = 1 - dblT * (0.002516 + dblT * 0.0000074)
    dblEE(0) = 1
    For intI = 1 To 2
        dblEE(intI) = dblEE(intI - 1) * dblE
    Next
    udtMoon.EclLong = 0
    udtMoon.DistanceGeo = 0
    udtMoon.EclDecl = 0
    For intI = 0 To 59
        dblPar = FracCirc(dblD * lngMoonLR(0, intI) + dblM * lngMoonLR(1, intI) + dblMp * lngMoonLR(2, intI) + dblF * lngMoonLR(3, intI))
        udtMoon.EclLong = udtMoon.EclLong + lngMoonLR(4, intI) * dblEE(Abs(lngMoonLR(1, intI))) * sinD(dblPar)
        udtMoon.DistanceGeo = udtMoon.DistanceGeo + lngMoonLR(5, intI) * dblEE(Abs(lngMoonLR(1, intI))) * cosD(dblPar)
        dblPar = FracCirc(dblD * lngMoonB(0, intI) + dblM * lngMoonB(1, intI) + dblMp * lngMoonB(2, intI) + dblF * lngMoonB(3, intI))
        udtMoon.EclDecl = udtMoon.EclDecl + lngMoonB(4, intI) * dblEE(Abs(lngMoonB(1, intI))) * sinD(dblPar)
    Next
    udtMoon.EclLong = dblLp + (udtMoon.EclLong + 3958 * sinD(dblA1) + 1962 * sinD(dblLp - dblF) + 318 * sinD(dblA2)) / 1000000
    dblOmega = 125.04 - 1934.136 * dblT 'Used for nutation correction
    'Convert to apparent longitude correcting for abberation and nutation
    udtMoon.EclLong = FracCirc(udtMoon.EclLong - 0.00569 * Cos(dblD) - 0.00478 * sinD(dblOmega))
    udtMoon.EclDecl = (udtMoon.EclDecl - 2235 * sinD(dblLp) + 382 * sinD(dblA3) + 175 * sinD(dblA1 - dblF) + 175 * sinD(dblA1 + dblF) + 127 * sinD(dblLp - dblMp) - 115 * sinD(dblLp + dblMp)) / 1000000
    udtMoon.DistanceGeo = 385000.56 + udtMoon.DistanceGeo / 1000
    Call cSun.ConvCoords(udtMoon, dblFix, udtWeatherNow)
    Call cSun.TrueSun(dblFix, udtSun)
    udtMoon.ElongationGeo = sinD(udtMoon.DeclGeo) * sinD(udtSun.DeclTopo) + cosD(udtMoon.DeclGeo) * cosD(udtSun.DeclGeo) * cosD(udtMoon.RAGeo - udtSun.RAGeo)
    If udtMoon.ElongationGeo > 0.999995 Then
        udtMoon.ElongationGeo = Sqr(Square(udtSun.RAGeo - udtMoon.RATopo) * cosD((udtMoon.DeclGeo + udtSun.DeclGeo) / 2) + Square(udtMoon.DeclGeo - udtSun.DeclGeo))
    ElseIf udtMoon.ElongationGeo <= -1 Then
        udtMoon.ElongationGeo = 180
    Else
        udtMoon.ElongationGeo = acosD(udtMoon.ElongationGeo)
    End If
    udtMoon.ElongationTopo = sinD(udtMoon.DeclTopo) * sinD(udtSun.DeclTopo) + cosD(udtMoon.DeclTopo) * cosD(udtSun.DeclTopo) * cosD(udtMoon.RATopo - udtSun.RATopo)
    If udtMoon.ElongationTopo > 0.999995 Then
        udtMoon.ElongationTopo = Sqr(Square(udtSun.RATopo - udtMoon.RATopo) * cosD((udtMoon.DeclTopo + udtSun.DeclTopo) / 2) + Square(udtMoon.DeclTopo - udtSun.DeclTopo))
    ElseIf udtMoon.ElongationTopo <= -1 Then
        udtMoon.ElongationTopo = 180
    Else
        udtMoon.ElongationTopo = acosD(udtMoon.ElongationTopo)
    End If
    udtMoon.Radius = asinD(3476 / 2 / udtMoon.DistanceTopo)
    udtMoon.Phase = (1 - Sqr(1 - Square(udtSun.DistanceTopo * sinD(udtMoon.ElongationTopo)) / (Square(udtSun.DistanceTopo) + Square(udtMoon.DistanceGeo) - 2 * udtSun.DistanceTopo * udtMoon.DistanceTopo * cosD(udtMoon.ElongationTopo)))) / 2
    udtMoon.Phase = (1 - Sqr(1 - Square(udtSun.DistanceTopo * sinD(udtMoon.ElongationTopo + (udtMoon.Phase - 1) * udtMoon.Radius / 2)) / (Square(udtSun.DistanceTopo) + Square(udtMoon.DistanceGeo) - 2 * udtSun.DistanceTopo * udtMoon.DistanceTopo * cosD(udtMoon.ElongationTopo + (udtMoon.Phase - 1) * udtMoon.Radius / 2)))) / 2
    If udtMoon.DistanceTopo > udtSun.DistanceTopo * cosD(udtMoon.ElongationTopo) Then udtMoon.Phase = 1 - udtMoon.Phase
End Sub
Friend Sub RiseSet(lngFix As Long, blnSet As Boolean, blnApparent As Boolean, dblRiseSetTime As Double, blnNeverRises As Boolean, blnNeverSets As Boolean, Optional blnSeaLevelHorizon As Boolean = False, Optional dblAltitude As Double = 0)
    'Calculate Moon Rise or set
    Dim cLST As New clsLST
    Dim dblTheta0 As Double
    Dim udtMoon(3) As HBody
    Dim dblH0 As Double
    Dim dblH As Double
    Dim dblHH As Double
    Dim dblY(3) As Double
    Dim dblAlpha As Double
    Dim dblDelta As Double
    Dim dblRadius As Double
    Dim cDeltaT As New clsDeltaT
    Dim dblDip As Double
    Dim udtWeatherNow As WeatherNow
    Dim cSun As New clsSun
    Dim blnClose As Boolean
    Dim dblStep As Double
    Dim dblMidday As Double
    Dim dblFix2 As Double
    Dim intI As Double
    Dim intDayShift As Integer
    
    dblStep = 1
    intDayShift = 0
    Call cSun.Meridian(lngFix, True, dblMidday)
    dblMidday = dblMidday - lngFix
    blnClose = False
    dblDip = dblAltitude
    dblRiseSetTime = dblMidday
    While Not blnClose
        dblFix2 = dblRiseSetTime
        dblTheta0 = cLST.FixToLST(lngFix + dblFix2, False)
        For intI = 0 To 2
            Call TrueMoon(lngFix + dblFix2 + (intI - 1) * dblStep, udtMoon(intI))
        Next
        Call frmWeather.GetWeatherNow(lngFix + dblFix2, 0, udtWeatherNow)
        If blnApparent Then
            dblDip = -udtMoon(1).Radius - cSun.Refract(0, False, udtWeatherNow) + dblAltitude
            If blnSeaLevelHorizon Then dblDip = dblDip - cSun.SeaLevelHorizon(udtWeatherNow)
        End If
        dblH0 = sinD(dblDip) - sinD(udtLocation.dblLatitude) * sinD(udtMoon(1).DeclTopo) / cosD(udtLocation.dblLatitude) / cosD(udtMoon(1).DeclTopo)
        blnNeverRises = dblH0 > 1
        blnNeverSets = dblH0 < -1
        If dblH0 < -1 Then
            dblH0 = -1
        ElseIf dblH0 > 1 Then
            dblH0 = 1
        End If
        If blnSet Then
            dblRiseSetTime = FracCirc(udtMoon(1).RATopo - dblTheta0 + acosD(dblH0)) / 360 + dblFix2 + intDayShift
        Else
            dblRiseSetTime = FracCirc(udtMoon(1).RATopo - dblTheta0 - acosD(dblH0)) / 360 + dblFix2 + intDayShift
        End If
        If dblStep = 1 Then
            While dblRiseSetTime < -0.518 + dblMidday
                dblRiseSetTime = dblRiseSetTime + 1
                intDayShift = intDayShift + 1
            Wend
            While dblRiseSetTime >= 0.518 + dblMidday
                dblRiseSetTime = dblRiseSetTime - 1
                intDayShift = intDayShift - 1
            Wend
        End If
        If Abs(udtMoon(2).EclLong - udtMoon(0).EclLong) > 0.1 And Abs(udtMoon(2).EclLong - udtMoon(0).EclLong) < 359.9 Then
            dblStep = dblStep / 20
        Else
            blnClose = True
        End If
    Wend
    If Abs(dblH0) > 0.98 Then
        Call frmWeather.GetWeatherNow(lngFix + dblRiseSetTime, 0, udtWeatherNow)
        Call TrueMoon(lngFix + dblRiseSetTime, udtMoon(1))
        If blnApparent Then
            dblDip = -udtMoon(1).Radius - cSun.Refract(0, False, udtWeatherNow) + dblAltitude
            If blnSeaLevelHorizon Then dblDip = dblDip - cSun.SeaLevelHorizon(udtWeatherNow)
        Else
            dblDip = 0
        End If
        dblH0 = sinD(dblDip) - sinD(udtLocation.dblLatitude) * sinD(udtMoon(1).DeclTopo) / cosD(udtLocation.dblLatitude) / cosD(udtMoon(1).DeclTopo)
        blnNeverRises = dblH0 > 1
        blnNeverSets = dblH0 < -1
        If dblH0 < -1 Then
            dblH0 = -1
        ElseIf dblH0 > 1 Then
            dblH0 = 1
        End If
        If blnSet Then
            dblRiseSetTime = (dblAlpha - dblTheta0 + acosD(dblH0)) / 360
        Else
            dblRiseSetTime = FracCirc(dblAlpha - dblTheta0 - acosD(dblH0)) / 360
        End If
    Else
'        Call TrueMoon(Int(dblFix) + dblRiseSetTime, udtMoon(0))
'        Debug.Print -dblDip, udtMoon(0).AltTopo
        dblY(0) = udtMoon(0).RATopo
        dblY(1) = udtMoon(1).RATopo
        dblY(2) = udtMoon(2).RATopo
        dblAlpha = Interp(dblY, (dblRiseSetTime - dblFix2) / dblStep + 1, 3)
        dblY(0) = udtMoon(0).DeclTopo
        dblY(1) = udtMoon(1).DeclTopo
        dblY(2) = udtMoon(2).DeclTopo
        dblDelta = Interp(dblY, (dblRiseSetTime - dblFix2) / dblStep + 1, 3)
        dblY(0) = udtMoon(0).Radius
        dblY(1) = udtMoon(1).Radius
        dblY(2) = udtMoon(2).Radius
        dblRadius = Interp(dblY, (dblRiseSetTime - dblFix2) / dblStep + 1, 3)
        dblH = cLST.FixToLST(lngFix + dblRiseSetTime, False)
        dblHH = asinD(sinD(udtLocation.dblLatitude) * sinD(dblDelta) + cosD(udtLocation.dblLatitude) * cosD(dblDelta) * cosD(dblH - dblAlpha))
        dblRiseSetTime = dblRiseSetTime + (dblHH - dblDip) / 360 / cosD(dblDelta) / cosD(udtLocation.dblLatitude) / sinD(dblH - dblAlpha)
    End If
    dblRiseSetTime = dblRiseSetTime + lngFix
'    Call TrueMoon(Int(dblFix) + dblRiseSetTime, udtMoon(0))
'    Debug.Print -dblDip, udtMoon(0).AltTopo
End Sub
Friend Sub YallopQ(dblFix As Double, dblYallopQ As Double, blnNew As Boolean, Optional dblBestTime As Double, Optional dblARCV As Double, Optional dblWp As Double)
    Dim cSun As New clsSun
    Dim udtMoon As HBody
    Dim udtSun As HBody
    Dim blnSet As Boolean
    Dim lngFix As Long
    Dim dblMoonRiseSetTime As Double
    Dim dblSunRiseSetTime As Double
    Dim blnSunNeverRises As Boolean
    Dim blnSunNeverSets As Boolean
    Dim blnMoonNeverRises As Boolean
    Dim blnMoonNeverSets As Boolean
    
    lngFix = frmDate.ToLngDate(dblFix)
    blnSet = IsNew(lngFix)
    Call cSun.RiseSet(lngFix, blnSet, dblSunRiseSetTime, False, blnSunNeverRises, blnSunNeverSets)
    Call RiseSet(lngFix, blnSet, False, dblMoonRiseSetTime, blnMoonNeverRises, blnMoonNeverSets)
    If dblMoonRiseSetTime < dblSunRiseSetTime - 0.2 And blnSet Then
        Call RiseSet(lngFix + 1, blnSet, False, dblMoonRiseSetTime, blnMoonNeverRises, blnMoonNeverSets)
    ElseIf dblMoonRiseSetTime > dblSunRiseSetTime + 0.2 And Not blnSet Then
        Call RiseSet(lngFix - 1, blnSet, False, dblMoonRiseSetTime, blnMoonNeverRises, blnMoonNeverSets)
    End If
    dblBestTime = (5 * dblSunRiseSetTime + 4 * dblMoonRiseSetTime) / 9
    Call cSun.TrueSun(dblBestTime, udtSun)
    Call TrueMoon(dblBestTime, udtMoon)
    dblARCV = udtMoon.AltGeo - udtSun.AltGeo
    dblWp = ((1 - cosD(udtMoon.ElongationGeo)) * udtMoon.Radius) * 60
    dblYallopQ = (dblARCV - 11.8371 + dblWp * (6.3226 - dblWp * (0.7319 - dblWp * 0.1018))) / 10
    blnNew = blnSet
End Sub
Friend Function YallopQText() As String
    Dim dblYallopQ As Double
    Dim blnNew As Boolean
    Dim dblBestTime As Double
    Dim dblARCV As Double
    Dim dblWp As Double
    Dim udtDateOut As MyDate
    Dim udtTimeOut As Mytime
    Dim blnValid As Boolean
    Dim strOldDate As String
    
    strOldDate = ""
    blnValid = True
    Call YallopQ(dblDate + dblTime, dblYallopQ, blnNew, dblBestTime, dblARCV, dblWp)
    YallopQText = "Yallop's q factor for "
    If blnNew Then
        YallopQText = YallopQText & "new"
    Else
        YallopQText = YallopQText & "old"
    End If
    Call frmMain.ToDateTime(dblBestTime, udtDateOut, udtDate.enmDateType, udtTimeOut, udtTime.enmTimeType, blnValid)
    YallopQText = YallopQText & " Moon" & vbCrLf & frmDate.WriteDateTime(udtDateOut, udtTimeOut, strOldDate)
    YallopQText = YallopQText & "Geocentric altitude difference " & strAngle(dblARCV) & vbCrLf
    YallopQText = YallopQText & "Geocentric crescent width " & Format(dblWp, "#0.000") & "'" & vbCrLf
    YallopQText = YallopQText & "Yallop's q value " & Format(dblYallopQ, "#0.000")
End Function
Friend Sub Rambam(dblFix As Double, dblOrechRishon As Double, dblKeshetReiya As Double, blnNew As Boolean, Optional dblBestTime As Double)
    Dim dblSunRiseSetTime As Double
    Dim dblMoonRiseSetTime As Double
    Dim blnSunNeverRises As Boolean
    Dim blnSunNeverSets As Boolean
    Dim blnMoonNeverRises As Boolean
    Dim blnMoonNeverSets As Boolean
    Dim lngFix As Long
    Dim udtMoon As HBody
    Dim udtSun As HBody
    Dim intFactor As Integer
    Dim cSun As New clsSun

    lngFix = frmDate.ToLngDate(dblFix)
    blnNew = IsNew(lngFix)
    Call cSun.RiseSet(lngFix, blnNew, dblSunRiseSetTime, False, blnSunNeverRises, blnSunNeverSets)
    Call RiseSet(lngFix, blnNew, False, dblMoonRiseSetTime, blnSunNeverRises, blnSunNeverSets)
    If dblMoonRiseSetTime < dblSunRiseSetTime - 0.2 And blnNew Then
        Call RiseSet(lngFix + 1, blnNew, False, dblMoonRiseSetTime, blnMoonNeverRises, blnMoonNeverSets)
        dblMoonRiseSetTime = dblMoonRiseSetTime + 1
    ElseIf dblMoonRiseSetTime > dblSunRiseSetTime + 0.2 And Not blnNew Then
        Call RiseSet(lngFix - 1, blnNew, False, dblMoonRiseSetTime, blnMoonNeverRises, blnMoonNeverSets)
        dblMoonRiseSetTime = dblMoonRiseSetTime - 1
    End If
    If blnNew Then
        intFactor = 1
    Else
        intFactor = -1
    End If
    dblKeshetReiya = FracCirc(intFactor * 360 * (dblMoonRiseSetTime - dblSunRiseSetTime))
    dblBestTime = dblSunRiseSetTime + intFactor * 1.38888888888889E-02
    Call TrueMoon(dblBestTime, udtMoon)
    Call cSun.TrueSun(dblBestTime, udtSun)
    dblOrechRishon = FracCirc(intFactor * (udtMoon.EclLong - udtSun.EclLong))
End Sub
Friend Function RambamText() As String
    Dim blnNew As Boolean
    Dim dblBestTime As Double
    Dim dblOR As Double
    Dim dblKR As Double
    Dim udtDateOut As MyDate
    Dim udtTimeOut As Mytime
    Dim blnValid As Boolean
    Dim strOldDate As String
    
    strOldDate = ""
    blnValid = True
    Call Rambam(dblDate + dblTime, dblOR, dblKR, blnNew, dblBestTime)
    RambamText = "Rambam's paramters for "
    If blnNew Then
        RambamText = RambamText & "new"
    Else
        RambamText = RambamText & "old"
    End If
    Call frmMain.ToDateTime(dblBestTime, udtDateOut, udtDate.enmDateType, udtTimeOut, udtTime.enmTimeType, blnValid)
    RambamText = RambamText & " Moon" & vbCrLf & frmDate.WriteDateTime(udtDateOut, udtTimeOut, strOldDate)
    RambamText = RambamText & "Orech Rishon (Geocentric ecliptic longitude difference) " & strAngle(dblOR) & vbCrLf
    RambamText = RambamText & "Keshet Reiya (proportional to Geometric lag) " & strAngle(dblKR) & vbCrLf
    RambamText = RambamText & "Sum (OR + KR) " & strAngle(dblOR + dblKR)
End Function
Friend Sub LagPhaseDistance(dblFix As Double, blnNew As Boolean, dblLag As Double, dblPhase As Double, dblDistance As Double, Optional dblSunset As Double)
    Dim dblSunRiseSetTime As Double
    Dim dblMoonRiseSetTime As Double
    Dim blnSunNeverRises As Boolean
    Dim blnSunNeverSets As Boolean
    Dim blnMoonNeverRises As Boolean
    Dim blnMoonNeverSets As Boolean
    Dim lngFix As Long
    Dim udtMoon As HBody
    Dim udtSun As HBody
    Dim intFactor As Integer
    Dim cSun As New clsSun

    lngFix = frmDate.ToLngDate(dblFix)
    blnNew = IsNew(lngFix)
    Call cSun.RiseSet(lngFix, blnNew, dblSunRiseSetTime, True, blnSunNeverRises, blnSunNeverSets, 0, True)
    Call RiseSet(lngFix, blnNew, True, dblMoonRiseSetTime, blnSunNeverRises, blnSunNeverSets, True)
    If dblMoonRiseSetTime < dblSunRiseSetTime - 0.2 And blnNew Then
        Call RiseSet(lngFix + 1, blnNew, False, dblMoonRiseSetTime, blnMoonNeverRises, blnMoonNeverSets)
    ElseIf dblMoonRiseSetTime > dblSunRiseSetTime + 0.2 And Not blnNew Then
        Call RiseSet(lngFix - 1, blnNew, False, dblMoonRiseSetTime, blnMoonNeverRises, blnMoonNeverSets)
    End If
    If blnNew Then
        intFactor = 1
    Else
        intFactor = -1
    End If
    dblLag = intFactor * 1440 * (dblMoonRiseSetTime - dblSunRiseSetTime)
    dblSunset = dblSunRiseSetTime
    Call TrueMoon(dblSunset, udtMoon)
    dblPhase = udtMoon.Phase * 100
    dblDistance = udtMoon.DistanceTopo
End Sub
Friend Function LagPhaseDistanceText() As String
    Dim dblLag As Double
    Dim dblPhase As Double
    Dim dblDistance As Double
    Dim dblSunset As Double
    Dim dblAngle As Double
    Dim dblSubtend As Double
    Dim dblAltitude As Double
    Dim dblAzimuth As Double
    Dim blnNew As Boolean
    Dim udtDateOut As MyDate
    Dim udtTimeOut As Mytime
    Dim blnValid As Boolean
    Dim strOldDate As String
    
    strOldDate = ""
    blnValid = True
    Call LagPhaseDistance(dblDate + dblTime, blnNew, dblLag, dblPhase, dblDistance, dblSunset)
    If blnNew Then
        LagPhaseDistanceText = "New "
    Else
        LagPhaseDistanceText = "Old "
    End If
    LagPhaseDistanceText = LagPhaseDistanceText & "Moon's Lag time, phase and distance" & vbCrLf
    If blnNew Then
        LagPhaseDistanceText = LagPhaseDistanceText & "Apparent sunset"
    Else
        LagPhaseDistanceText = LagPhaseDistanceText & "Apparent sunrise"
    End If
    Call frmMain.ToDateTime(dblSunset, udtDateOut, udtDate.enmDateType, udtTimeOut, udtTime.enmTimeType, blnValid)
    LagPhaseDistanceText = LagPhaseDistanceText & " over sea level horizon " & vbCrLf & frmDate.WriteDateTime(udtDateOut, udtTimeOut, strOldDate)
    LagPhaseDistanceText = LagPhaseDistanceText & "Apparent lag time " & WriteDMSTime(dblLag / 60) & vbCrLf
    LagPhaseDistanceText = LagPhaseDistanceText & "Phase " & Format(dblPhase, "##0.00") & "%" & vbCrLf
    LagPhaseDistanceText = LagPhaseDistanceText & "Topocentric distance (Earth's surface to Moon's center) " & Int(dblDistance + 0.5) & "km" & vbCrLf
    Call BrightLimb(dblSunset, dblAngle, dblSubtend, dblAltitude, dblAzimuth)
    LagPhaseDistanceText = LagPhaseDistanceText & "Zenith angle " & Format(dblAngle, "##0") & "" & vbCrLf
    LagPhaseDistanceText = LagPhaseDistanceText & "Crescent subtends an angle of " & Format(dblSubtend, "##0") & ""
End Function
Friend Function EclipticCoordinates() As String
    Dim udtMoon As HBody
    Dim udtDms As DMS
    Dim cDMS As New clsDMS
    
    Call TrueMoon(dblDate + dblTime, udtMoon)
    Call cDMS.ToDMS(udtMoon.EclLong, udtDms)
    EclipticCoordinates = "Moon's geocentric ecliptic coordinates" & vbCrLf & "Longitude " & udtDms.intDegreesHours & "" & udtDms.intMinutes & "'" & Int(udtDms.dblSeconds) & """" & vbCrLf
    Call cDMS.ToDMS(udtMoon.EclDecl, udtDms)
    EclipticCoordinates = EclipticCoordinates & "Declination "
    If Not udtDms.blnSign Then EclipticCoordinates = EclipticCoordinates & "-"
    EclipticCoordinates = EclipticCoordinates & udtDms.intDegreesHours & "" & udtDms.intMinutes & "'" & Int(udtDms.dblSeconds) & """"
End Function
Friend Function EquatorialCoordinates() As String
    Dim udtMoon As HBody
    Dim udtDms As DMS
    Dim cDMS As New clsDMS
    
    Call TrueMoon(dblDate + dblTime, udtMoon)
    Call cDMS.ToDMS(udtMoon.RAGeo, udtDms)
    EquatorialCoordinates = "Moon's geocentric equatorial coordinates" & vbCrLf & "Right ascension " & udtDms.intDegreesHours & "" & udtDms.intMinutes & "'" & Int(udtDms.dblSeconds) & """" & vbCrLf
    Call cDMS.ToDMS(udtMoon.RAGeo / 15, udtDms)
    EquatorialCoordinates = EquatorialCoordinates & "              = " & udtDms.intDegreesHours & "h" & udtDms.intMinutes & "m" & Int(udtDms.dblSeconds) & "s" & vbCrLf
    Call cDMS.ToDMS(udtMoon.DeclGeo, udtDms)
    EquatorialCoordinates = EquatorialCoordinates & "Declination "
    If Not udtDms.blnSign Then EquatorialCoordinates = EquatorialCoordinates & "-"
    EquatorialCoordinates = EquatorialCoordinates & udtDms.intDegreesHours & "" & udtDms.intMinutes & "'" & Int(udtDms.dblSeconds) & """" & vbCrLf
    Call cDMS.ToDMS(udtMoon.RATopo, udtDms)
    EquatorialCoordinates = EquatorialCoordinates & "Moon's topocentric equatorial coordinates" & vbCrLf & "Right ascension " & udtDms.intDegreesHours & "" & udtDms.intMinutes & "'" & Int(udtDms.dblSeconds) & """" & vbCrLf
    Call cDMS.ToDMS(udtMoon.RATopo / 15, udtDms)
    EquatorialCoordinates = EquatorialCoordinates & "              = " & udtDms.intDegreesHours & "h" & udtDms.intMinutes & "m" & Int(udtDms.dblSeconds) & "s" & vbCrLf
    Call cDMS.ToDMS(udtMoon.DeclTopo, udtDms)
    EquatorialCoordinates = EquatorialCoordinates & "Declination "
    If Not udtDms.blnSign Then EquatorialCoordinates = EquatorialCoordinates & "-"
    EquatorialCoordinates = EquatorialCoordinates & udtDms.intDegreesHours & "" & udtDms.intMinutes & "'" & Int(udtDms.dblSeconds) & """"
End Function
Friend Function HorizonCoordinates() As String
    Dim udtMoon As HBody
    Dim udtDms As DMS
    Dim cDMS As New clsDMS
    
    Call TrueMoon(dblDate + dblTime, udtMoon)
    Call cDMS.ToDMS(udtMoon.AzGeo, udtDms)
    HorizonCoordinates = "Moon's geocentric horizon coordinates" & vbCrLf & "Azimuth " & udtDms.intDegreesHours & "" & udtDms.intMinutes & "'" & Int(udtDms.dblSeconds) & """" & vbCrLf
    Call cDMS.ToDMS(udtMoon.AltGeo, udtDms)
    HorizonCoordinates = HorizonCoordinates & "Altitude "
    If Not udtDms.blnSign Then HorizonCoordinates = HorizonCoordinates & "-"
    HorizonCoordinates = HorizonCoordinates & udtDms.intDegreesHours & "" & udtDms.intMinutes & "'" & Int(udtDms.dblSeconds) & """" & vbCrLf
    Call cDMS.ToDMS(udtMoon.AzTopo, udtDms)
    HorizonCoordinates = HorizonCoordinates & "Moon's topocentric horizon coordinates" & vbCrLf & "Azimuth " & udtDms.intDegreesHours & "" & udtDms.intMinutes & "'" & Int(udtDms.dblSeconds) & """" & vbCrLf
    Call cDMS.ToDMS(udtMoon.AltTopo, udtDms)
    HorizonCoordinates = HorizonCoordinates & "Altitude "
    If Not udtDms.blnSign Then HorizonCoordinates = HorizonCoordinates & "-"
    HorizonCoordinates = HorizonCoordinates & udtDms.intDegreesHours & "" & udtDms.intMinutes & "'" & Int(udtDms.dblSeconds) & """" & vbCrLf
    If udtMoon.AltRefr <> -90 Then
        Call cDMS.ToDMS(udtMoon.AltRefr, udtDms)
        HorizonCoordinates = HorizonCoordinates & "After correction for refraction" & vbCrLf & "Altitude "
        If Not udtDms.blnSign Then HorizonCoordinates = HorizonCoordinates & "-"
        HorizonCoordinates = HorizonCoordinates & udtDms.intDegreesHours & "" & udtDms.intMinutes & "'" & Int(udtDms.dblSeconds) & """"
    Else
        HorizonCoordinates = HorizonCoordinates & "The Moon is well below the horizon so refraction cannot be calculated."
    End If
End Function
Friend Function DistSize() As String
    Dim udtMoon As HBody
    Dim udtDms As DMS
    Dim cDMS As New clsDMS
    
    Call TrueMoon(dblDate + dblTime, udtMoon)
    DistSize = "Moon's distance and size" & vbCrLf & "Geocentric distance " & Format(udtMoon.DistanceGeo, "000000") & " km" & vbCrLf
    DistSize = DistSize & "Topocentric distance " & Format(udtMoon.DistanceTopo, "000000") & " km" & vbCrLf
    Call cDMS.ToDMS(udtMoon.Radius, udtDms)
    DistSize = DistSize & "Topocentric radius " & udtDms.intMinutes & "'" & Int(udtDms.dblSeconds) & """"
End Function
Friend Function Moonrise() As String
    Dim dblRiseSetTime As Double
    Dim blnNeverSets As Boolean
    Dim blnNeverRises As Boolean
    Dim intDays As Integer
    Dim udtDateOut As MyDate
    Dim udtTimeOut As Mytime
    Dim lngFix As Long
    Dim udtDms As DMS
    Dim blnValid As Boolean
    Dim cDMS As New clsDMS
    Dim strOldDate As String
    
    strOldDate = ""
    lngFix = frmDate.ToLngDate(dblDate + dblTime)
    blnValid = True
    Call RiseSet(lngFix, False, False, dblRiseSetTime, blnNeverRises, blnNeverSets)
    If blnNeverSets Then
        Moonrise = "No geometric moonset" & vbCrLf & "Minimum altitude at "
    ElseIf blnNeverRises Then
        Moonrise = "No geometric moonrise" & vbCrLf & "Maximum altitude at "
    Else
        Moonrise = "Geometric moonrise "
    End If
    Call frmMain.ToDateTime(dblRiseSetTime, udtDateOut, udtDate.enmDateType, udtTimeOut, udtTime.enmTimeType, blnValid)
    Moonrise = Moonrise & frmDate.WriteDateTime(udtDateOut, udtTimeOut, strOldDate)
    If Not blnNeverSets Then
        Call RiseSet(lngFix, False, True, dblRiseSetTime, blnNeverRises, blnNeverSets)
        If blnNeverSets Then
            Moonrise = Moonrise & vbCrLf & "No apparent moonset" & vbCrLf & "Minimum altitude at "
        ElseIf blnNeverRises Then
            Moonrise = Moonrise & vbCrLf & "No apparent moonrise" & vbCrLf & "Maximum altitude at "
        Else
            Moonrise = Moonrise & vbCrLf & "Apparent moonrise "
        End If
        Call frmMain.ToDateTime(dblRiseSetTime, udtDateOut, udtDate.enmDateType, udtTimeOut, udtTime.enmTimeType, blnValid)
        Moonrise = Moonrise & frmDate.WriteDateTime(udtDateOut, udtTimeOut, strOldDate)
    End If
End Function
Friend Function MoonSet() As String
    Dim dblRiseSetTime As Double
    Dim blnNeverSets As Boolean
    Dim blnNeverRises As Boolean
    Dim intDays As Integer
    Dim lngFix As Long
    Dim udtDateOut As MyDate
    Dim udtTimeOut As Mytime
    Dim udtDms As DMS
    Dim blnValid As Boolean
    Dim cDMS As New clsDMS
    Dim strOldDate As String
    
    strOldDate = ""
    lngFix = frmDate.ToLngDate(dblDate + dblTime)
    blnValid = True
    Call RiseSet(lngFix, True, False, dblRiseSetTime, blnNeverRises, blnNeverSets)
    If blnNeverSets Then
        MoonSet = "No geometric moonset" & vbCrLf & "Minimum altitude at "
    ElseIf blnNeverRises Then
        MoonSet = "No geometric moonrise" & vbCrLf & "Maximum altitude at "
    Else
        MoonSet = "Geometric moonset "
    End If
    Call frmMain.ToDateTime(dblRiseSetTime, udtDateOut, udtDate.enmDateType, udtTimeOut, udtTime.enmTimeType, blnValid)
    MoonSet = MoonSet & frmDate.WriteDateTime(udtDateOut, udtTimeOut, strOldDate)
    If Not blnNeverSets Then
        Call RiseSet(lngFix, True, True, dblRiseSetTime, blnNeverRises, blnNeverSets)
        If blnNeverSets Then
            MoonSet = MoonSet & vbCrLf & "No apparent moonset" & vbCrLf & "Minimum altitude at "
        ElseIf blnNeverRises Then
            MoonSet = MoonSet & vbCrLf & "No apparent moonrise" & vbCrLf & "Maximum altitude at "
        Else
            MoonSet = MoonSet & vbCrLf & "Apparent moonset "
        End If
        Call frmMain.ToDateTime(dblRiseSetTime, udtDateOut, udtDate.enmDateType, udtTimeOut, udtTime.enmTimeType, blnValid)
        MoonSet = MoonSet & frmDate.WriteDateTime(udtDateOut, udtTimeOut, strOldDate)
    End If
End Function
Friend Function strAngle(dblAngle As Double) As String
    Dim cDMS As New clsDMS
    Dim udtDms As DMS
    
    Call cDMS.ToDMS(dblAngle + 1 / 7200 * Sgn(dblAngle), udtDms)
    If dblAngle < -1 / 7200 And udtDms.intDegreesHours = 0 Then
        strAngle = "-"
    Else
        strAngle = ""
    End If
    strAngle = strAngle & udtDms.intDegreesHours & "" & udtDms.intMinutes & "'" & Int(udtDms.dblSeconds) & """"
End Function
Friend Function strTime(dblTime As Double, strOldDate As String) As String
    Dim udtDateOut As MyDate
    Dim udtTimeOut As Mytime
    Dim blnValid As Boolean
    
    blnValid = True
    Call frmMain.ToDateTime(dblTime, udtDateOut, udtDate.enmDateType, udtTimeOut, udtTime.enmTimeType, blnValid)
    If blnValid Then
        strTime = frmDate.WriteDateTime(udtDateOut, udtTimeOut, strOldDate)
    Else
        strTime = "Date out of range."
    End If
End Function
Friend Sub Visibility(dblFix As Double, blnNew As Boolean, dblYallopQ As Double, dblT() As Double, blnEasy As Boolean, blnNotVisible As Boolean, dblProbVis As Double)
    Dim dblAltitude As Double
    Dim blnNeverRises As Boolean
    Dim blnNeverSets As Boolean
    Dim lngFix As Long
    Dim cSun As New clsSun
    Dim dblBestTime As Double
    Dim udtSun(1) As HBody
    
    lngFix = frmDate.ToLngDate(dblFix)
    Call YallopQ(dblFix, dblYallopQ, blnNew, dblBestTime)
    blnNotVisible = dblYallopQ < -0.1
    blnEasy = dblYallopQ > 1.85
    If blnNotVisible Then
        dblT(0) = dblBestTime
        dblT(1) = dblBestTime
        dblT(2) = dblBestTime
        dblT(3) = dblBestTime
        dblProbVis = 0
    ElseIf blnEasy Then
        dblProbVis = 1
    Else
        If blnNew Then
            dblAltitude = -8.0581 + dblYallopQ * (14.151 - dblYallopQ * (10.804 - dblYallopQ * 3.4415))
        Else
            dblAltitude = -6.8727 + dblYallopQ * (18.598 - dblYallopQ * (17.641 - dblYallopQ * 5.7615))
        End If
        Call cSun.RiseSet(lngFix, blnNew, dblT(0), False, blnNeverRises, blnNeverSets, dblAltitude + 2.62)
        Call cSun.RiseSet(lngFix, blnNew, dblT(1), False, blnNeverRises, blnNeverSets, dblAltitude - 2.62)
        Call RiseSet(lngFix, blnNew, False, dblT(2), blnNeverRises, blnNeverSets, False, 2.8 * Exp(-4 * dblYallopQ) + 3.3)
        Call RiseSet(lngFix, blnNew, False, dblT(3), blnNeverRises, blnNeverSets, False, 2.8 * Exp(-4 * dblYallopQ) - 0.7)
        Call cSun.TrueSun(lngFix + (dblT(0) + dblT(1)) / 2, udtSun(0))
        Call cSun.TrueSun(lngFix + (dblT(2) + dblT(3)) / 2, udtSun(1))
        dblProbVis = Prob((udtSun(0).AltTopo - udtSun(1).AltTopo) / 1.52)
        If dblT(0) < dblT(3) And Not blnNew Or dblT(0) > dblT(3) And blnNew Then
            dblT(0) = dblBestTime
            blnNotVisible = True
        End If
    End If
End Sub


Friend Function FirstVisible(dblFix As Double) As String
    Dim dblYallopQ As Double
    Dim dblT(3) As Double
    Dim blnNotVisible As Boolean
    Dim blnEasy As Boolean
    Dim blnNew As Boolean
    Dim dblAltitude As Double
    Dim blnNeverRises As Boolean
    Dim blnNeverSets As Boolean
    Dim cSun As New clsSun
    Dim dblBestTime As Double
    Dim udtSun(1) As HBody
    Dim dblProbVis As Double
    Dim strOldDate As String
    
    strOldDate = ""
    Call Visibility(dblFix, blnNew, dblYallopQ, dblT, blnEasy, blnNotVisible, dblProbVis)
    If blnNew Then
        FirstVisible = "The new Moon "
    Else
        FirstVisible = "The old Moon "
    End If
    If blnNotVisible Then
        FirstVisible = FirstVisible & "is not visible to the naked eye"
        If dblYallopQ > -0.23 Then
            FirstVisible = FirstVisible & " but may be visible with a telescope" & vbCrLf & "Best visibility at " & strTime(dblT(0), strOldDate)
        Else
            FirstVisible = FirstVisible & " or with a telescope"
        End If
    ElseIf blnEasy Then
        FirstVisible = FirstVisible & "is clearly visible for an extended period"
    Else
        If blnNew Then
            If dblT(2) < dblT(1) Then
                FirstVisible = FirstVisible & "first appears between: " & strTime(dblT(0), strOldDate) & " and " & strTime(dblT(3), strOldDate)
            Else
                FirstVisible = FirstVisible & "first appears between: " & strTime(dblT(0), strOldDate) & " and " & strTime(dblT(1), strOldDate) & vbCrLf
                FirstVisible = FirstVisible & "and fades between: " & strTime(dblT(2), strOldDate) & " and " & strTime(dblT(3), strOldDate)
            End If
        Else
            If dblT(1) < dblT(2) Then
                FirstVisible = FirstVisible & "first appears between: " & strTime(dblT(3), strOldDate) & " and " & strTime(dblT(0), strOldDate)
            Else
                FirstVisible = FirstVisible & "first appears between: " & strTime(dblT(3), strOldDate) & " and " & strTime(dblT(2), strOldDate) & vbCrLf
                FirstVisible = FirstVisible & "and fades between: " & strTime(dblT(1), strOldDate) & " and " & strTime(dblT(0), strOldDate)
            End If
        End If
        FirstVisible = FirstVisible & vbCrLf & "with 95% confidence provided that it is not cloudy" & vbCrLf
    End If
    If dblProbVis >= 0.01 And dblProbVis <= 0.99 Then
        FirstVisible = FirstVisible & " Probability of visibility if clear: " & Format(100 * dblProbVis, "##0") & "%"
    ElseIf dblProbVis >= 0.001 And dblProbVis <= 0.999 Then
        FirstVisible = FirstVisible & " Probability of visibility if clear: " & Format(100 * dblProbVis, "##0.0") & "%"
    Else
        FirstVisible = FirstVisible & " Probability of visibility if clear: " & Format(100 * dblProbVis, "##0.00") & "%"
    End If
    If Not blnEasy And (Not blnNotVisible Or dblYallopQ > -0.23) Then
        If Abs(udtLocation.dblLatitude > 62) Then
            FirstVisible = FirstVisible & vbCrLf & "Graph not implemented for high latitudes."
        Else
            If Not frmGraph.Visible Then frmGraph.Show
            frmGraph.Form_Resize
        End If
    End If
End Function
Friend Sub BrightLimb(dblFix As Double, dblAngle As Double, dblSubtend As Double, dblAltitude As Double, dblAzimuth As Double)
'Calculate the appearence of the bright limb of the Moon (corrected for refraction)
    Dim udtMoon As HBody
    Dim udtSun As HBody
    Dim dblOffset As Double
    Dim dblChi As Double 'Tilt of limb relative to Moon's pole
    Dim dblQ As Double 'Parallactic angle
    Dim udtWeatherNow As WeatherNow
    Dim cSun As New clsSun
    
    Call TrueMoon(dblFix, udtMoon)
    Call cSun.TrueSun(dblFix, udtSun)
    dblChi = 360 - atn2D(cosD(udtSun.DeclTopo) * sinD(udtSun.RATopo - udtMoon.RATopo), sinD(udtSun.DeclTopo) * cosD(udtMoon.DeclTopo) - cosD(udtSun.DeclTopo) * sinD(udtMoon.DeclTopo) * cosD(udtSun.RATopo - udtMoon.RATopo))
    dblQ = atn2D(sinD(udtMoon.HourAngTopo), tanD(udtLocation.dblLatitude) * cosD(udtMoon.DeclTopo) - sinD(udtMoon.DeclTopo) * cosD(udtMoon.HourAngTopo))
    dblAngle = FracCirc(180 - dblChi - dblQ)
    If dblAngle > 180 Then dblAngle = dblAngle - 360
    dblOffset = udtMoon.Radius * (1 - udtMoon.Phase) 'Offset to middle of crescent (or other phase)
    dblSubtend = 185 - 1460000000# * (udtMoon.ElongationTopo + 25) ^ -4.702
    dblAltitude = udtMoon.AltTopo - dblOffset * cosD(dblAngle)
    Call frmWeather.GetWeatherNow(dblFix, 0, udtWeatherNow)
    dblAltitude = dblAltitude + cSun.Refract(dblAltitude, True, udtWeatherNow)
    dblAzimuth = udtMoon.AzTopo + dblOffset * sinD(dblAngle)
End Sub
Function IsNew(lngFix As Long) As Boolean
'Find out if the geocentric Moon is new or old at true midday
    Dim cSun As New clsSun
    Dim dblMidday As Double
    Dim udtMoon As HBody
    Dim udtSun As HBody
    
    Call cSun.Meridian(lngFix, True, dblMidday)
    Call TrueMoon(dblMidday, udtMoon)
    Call cSun.TrueSun(dblMidday, udtSun)
    IsNew = FracCirc(udtMoon.EclLong - udtSun.EclLong) < 180
End Function
Friend Function WriteDMSTime(dblHours As Double) As String
    Dim udtDms As DMS
    Dim cDMS As New clsDMS
    
    Call cDMS.ToDMS(dblHours, udtDms)
    If udtDms.intDegreesHours > 0 Then
        WriteDMSTime = udtDms.intDegreesHours & "h" & Format(udtDms.intMinutes, "00") & "m" & Format(udtDms.dblSeconds, "00") & "s"
    ElseIf udtDms.intMinutes > 0 Then
        WriteDMSTime = udtDms.intMinutes & "m" & Format(udtDms.dblSeconds, "00") & "s"
    Else
        WriteDMSTime = Format(udtDms.dblSeconds, "#0") & "s"
    End If
End Function
Friend Function NextNewMoonText() As String
    Dim lngNext As Long
    Dim udtDateOut As MyDate
    Dim blnValid As Boolean
    
    lngNext = NextNewMoon(dblDate + dblTime)
    Call frmDate.ToDate(lngNext, udtDate.enmDateType, udtDateOut, blnValid)
    NextNewMoonText = frmDate.WriteDate(udtDateOut)
End Function
Friend Function NextNewMoon(dblDate As Double) As Long
    Dim lngDate As Long
    Dim blnNew As Boolean
    Dim udtMoon As HBody
    Dim dblYallopQ As Double
    Dim dblT(5) As Double
    Dim blnEasy As Boolean
    Dim blnNotVisible As Boolean
    Dim dblProbVis As Double
    Dim intDays As Integer
    
    lngDate = frmDate.ToLngDate(dblDate)
    blnNew = IsNew(lngDate)
    Call TrueMoon(dblDate, udtMoon)
    If blnNew Then
        Call Visibility(dblDate - 1, blnNew, dblYallopQ, dblT, blnEasy, blnNotVisible, dblProbVis)
        If dblProbVis < 0.005 Then
            intDays = -1
            While dblProbVis < 0.005
                intDays = intDays + 1
                Call Visibility(dblDate + intDays, blnNew, dblYallopQ, dblT, blnEasy, blnNotVisible, dblProbVis)
            Wend
            NextNewMoon = lngDate + intDays
        Else
            intDays = (360 - udtMoon.ElongationTopo) / 360 * 28
            Call Visibility(dblDate + intDays, blnNew, dblYallopQ, dblT, blnEasy, blnNotVisible, dblProbVis)
            While Not blnNew Or dblProbVis < 0.005
                intDays = intDays + 1
                Call Visibility(dblDate + intDays, blnNew, dblYallopQ, dblT, blnEasy, blnNotVisible, dblProbVis)
            Wend
            NextNewMoon = lngDate + intDays
        End If
    Else
        If blnNew Then
            intDays = (360 - udtMoon.ElongationTopo) / 360 * 28
        Else
            intDays = udtMoon.ElongationTopo / 360 * 28
        End If
        Call Visibility(dblDate + intDays, blnNew, dblYallopQ, dblT, blnEasy, blnNotVisible, dblProbVis)
        While Not blnNew Or dblProbVis < 0.005
            intDays = intDays + 1
            Call Visibility(dblDate + intDays, blnNew, dblYallopQ, dblT, blnEasy, blnNotVisible, dblProbVis)
        Wend
        NextNewMoon = lngDate + intDays
    End If
End Function
Friend Sub DaysAfterRC()
'Determine the number of days after Rosh Chodesh that the Moon appears
'Not in use
    Dim udtDate As MyDate
    Dim intDARC As Integer
    Dim intYear As Integer
    Dim intMonth As Integer
    Dim blnNewVis As Boolean
    Dim dblFix As Double
    Dim dblYallopQ As Double
    Dim blnNew As Boolean
    Dim dblAltitude As Double
    Dim dblT(3) As Double
    Dim blnNeverRises As Boolean
    Dim blnNeverSets As Boolean
    Dim udtSun(1) As HBody
    Dim strProb As String
    Dim blnValid As Boolean
    Dim dblProbVis As Double
    Dim cHebrew As New clsHebrew
    Dim cSun As New clsSun
    
    Open "Output.txt" For Output Access Write As 2
    udtDate.enmDateType = Hebrew
    For intYear = 1 To 6000
        For intMonth = 1 To cHebrew.MonthsInHebrewYear(udtDate.intYear)
            udtDate.intYear = intYear
            udtDate.intMonth = intMonth
            udtDate.intDateOfMonth = 4
            intDARC = 5
            blnNewVis = True
            While blnNewVis
                intDARC = intDARC - 1
                Call cHebrew.fixed_hebrew(cHebrew.hebrew_fixed(udtDate) - 1, udtDate, blnValid)
                dblFix = cHebrew.hebrew_fixed(udtDate) + 0.5
                Call YallopQ(dblFix, dblYallopQ, blnNew)
                If Not blnNew Or dblYallopQ < -0.1 Then
                    blnNewVis = False
                ElseIf dblYallopQ < 1.85 Then
                    dblAltitude = -8.0581 + dblYallopQ * (14.151 - dblYallopQ * (10.804 - dblYallopQ * 3.4415))
                    Call cSun.RiseSet(Int(dblFix), blnNew, dblT(0), False, blnNeverRises, blnNeverSets, dblAltitude + 2.62)
                    Call cSun.RiseSet(Int(dblFix), blnNew, dblT(1), False, blnNeverRises, blnNeverSets, dblAltitude - 2.62)
                    Call RiseSet(Int(dblFix), blnNew, False, dblT(2), blnNeverRises, blnNeverSets, False, 2.8 * Exp(-4 * dblYallopQ) + 3.3)
                    Call RiseSet(Int(dblFix), blnNew, False, dblT(3), blnNeverRises, blnNeverSets, False, 2.8 * Exp(-4 * dblYallopQ) - 0.7)
                    Call cSun.TrueSun(Int(dblDate) + (dblT(0) + dblT(1)) / 2, udtSun(0))
                    Call cSun.TrueSun(Int(dblDate) + (dblT(2) + dblT(3)) / 2, udtSun(1))
                    dblProbVis = Prob((udtSun(0).AltTopo - udtSun(1).AltTopo) / 1.52)
                    If dblProbVis >= 0.01 And dblProbVis <= 0.99 Then
                        strProb = "P =" & Format(100 * dblProbVis, "##0") & "%"
                    ElseIf dblProbVis >= 0.001 And dblProbVis <= 0.999 Then
                        strProb = "P = " & Format(100 * dblProbVis, "##0.0") & "%"
                    ElseIf dblProbVis > 0.0001 Then
                        strProb = "P =" & Format(100 * dblProbVis, "##0.00") & "%"
                    Else
                        blnNewVis = False
                    End If
                Else
                    strProb = "P = 100.00"
                End If
            Wend
            If intDARC < 0 Then Print #2, cHebrew.MonthName(intMonth, intYear), intYear, intDARC; " days ", strProb
        Next
    Next
    Close 2
End Sub
Friend Function Magnitude(udtMoon As HBody) As Double
'Calculate the magnitude of the Moon as seen from above the atmosphere
    Dim dblRD As Double
    Dim dblI As Double
    
    'Moon's phase in degrees
    dblI = acosD(2 * udtMoon.Phase - 1)
    'Planet's distance from the Sun times distance to planet in AU^2
'    dblRD = Abs(sinD(udtMoon.ElongationTopo) * udtMoon.DistanceTopo / sinD(180 - udtMoon.ElongationTopo - dblI) * udtMoon.DistanceTopo / AU / AU)
    Magnitude = -12.73 + 5 * Log10(udtMoon.DistanceTopo / 384400) + 0.026 * dblI + 0.000000004 * Square(Square(dblI))
End Function
Friend Function MagText() As String
    Dim udtMoon As HBody
    
    Call TrueMoon(dblDate + dblTime, udtMoon)
    MagText = "The Moon's magnitude above the atmosphere " & Format(Magnitude(udtMoon), "###.00")
End Function
Sub NewMoonTable()
    'Make table of new Moons
    Dim intI As Integer
    Dim cMoon As New clsMoon
    Dim dblYallopQ As Double
    Dim dblT(5) As Double
    Dim blnEasy As Boolean
    Dim blnNotVisible As Boolean
    Dim dblProbVis As Double
    Dim blnValid As Boolean
    Dim blnNew As Boolean
    Dim dblLastDate As Double
    
    Open "Output.txt" For Output Access Write As 2
    udtDate.intDateOfMonth = 31
    udtDate.intMonth = 12
    udtDate.intYear = 701
    udtDate.blnCE = False
    udtDate.enmDateType = Julian
    For intI = 1 To 3720
        dblDate = frmDate.EvaluateDate(udtDate) + 1
        dblLastDate = dblDate
        dblDate = cMoon.NextNewMoon(dblDate)
        dblLastDate = dblDate - dblLastDate + 1
        Call frmDate.ToDate(CLng(dblDate), udtDate.enmDateType, udtDate, blnValid)
        Call cMoon.Visibility(dblDate, blnNew, dblYallopQ, dblT, blnEasy, blnNotVisible, dblProbVis)
        Print #2, frmDate.WriteDate(udtDate), " Probability " & Format(dblProbVis * 100, "##.00") & "%", dblLastDate
    Next
    Close 2
End Sub
