VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsSun"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit
'(C)Roy E. Hoffman 2000
'See Main.bas for terms and conditions of use
'Some of the algorithms from
'J. Meeus, "Astronomical Algorithms", 2nd Ed., William-Bell Inc., Richmond, VG, 1998, p. 87

Const TPI = 6.28318530717959
Const PI = 3.14159265358979
Const RADCON = 0.01745329252
Const AU = 149597870 'Astronomical unit in km
Friend Function MeanSun(dblFix As Double) As Double 'Longitude in degrees
    Dim dblT As Double
    
    dblT = ToJ2000(dblFix)
    MeanSun = FracCirc(280.46646 + dblT * (36000.76983 + 0.0003032 * dblT))
End Function
Friend Function Eps(dblT As Double) As Double 'Inclination of Earth's axis
    Dim dblU As Double 'Myriads of years
    
    If Abs(dblT) <= 100 Then
        dblU = dblT / 100
        Eps = (7.12 + dblU * (27.87 + dblU * (5.79 + dblU * 2.45)))
        Eps = (51.38 + dblU * (249.67 + dblU * (39.05 - dblU * Eps)))
        Eps = (1.55 - dblU * (1999.25 - dblU * Eps))
        Eps = 23.4392911111111 - dblU / 3600 * (4680.93 + dblU * Eps)
    Else
        Eps = 23.4392911111111 - dblT / 3600 * (46.815 + dblT * (0.00059 - dblT * 0.001813))
    End If
End Function
Friend Sub TrueSun(dblFix As Double, udtSun As HBody, Optional blnLowPre = False) 'Low accuracy method +/- 0.01 deg
    Dim dblL0 As Double 'Mean Sun
    Dim dblM As Double 'Mean anomaly
    Dim dblE As Double 'Eccentricity
    Dim dblC As Double 'Equation of center
    Dim dblNu As Double 'True anomaly
    Dim dblOmega As Double 'Correction to true equinox due to nutation
    Dim dblEps As Double 'Inclination of Earth's axis
    Dim dblT As Double 'Julian centuries from J2000
    Dim dblH As Double 'Hour angle
    Dim cLST As New clsLST
    Dim cPlanet As New clsPlanet
    
    dblT = ToJ2000(dblFix)
    dblOmega = 125.04 - 1934.136 * dblT 'Used for nutation correction
    dblEps = Eps(dblT) + 0.00256 * cosD(dblOmega)
    If blnLowPre Then
        dblL0 = MeanSun(dblFix)
        dblM = FracCirc(357.52911 + dblT * (35999.05029 - 0.0001537 * dblT))
        dblE = 0.016708634 - dblT * (0.000042037 + 0.0000001267 * dblT)
        dblC = (1.914602 - dblT * (0.004817 + 0.000014 * dblT)) * sinD(dblM) + (0.019993 - 0.000101 * dblT) * sinD(2 * dblM) + 0.000289 * sinD(3 * dblM)
        dblNu = dblM + dblC
        udtSun.EclLong = FracCirc(dblL0 + dblC - 0.00569 - 0.00478 * sinD(dblOmega)) 'Apparent ecliptic longitude
        udtSun.EclDecl = 0
        udtSun.DistanceGeo = 1.00001018 * (1 - Square(dblE)) / (1 + dblE * cosD(dblNu)) * 149597870 'in km
    Else
        If Not blnEarth Then Call cPlanet.ReadPlanet("Earth", udtEarth)
        blnEarth = True
        Call cPlanet.HelioPlanet("Earth", dblFix, udtSun.EclLong, udtSun.EclDecl, udtSun.DistanceGeo)
        udtSun.EclLong = FracCirc(180 + udtSun.EclLong - 0.00569 - 0.00478 * sinD(dblOmega))
        udtSun.EclDecl = -udtSun.EclDecl
    End If
    Call ConvCoords(udtSun, dblFix, udtWeatherNow)
    udtSun.Phase = 1
    udtSun.Radius = atn2D(1392000 / 2 / udtSun.DistanceTopo, 1) 'Apparent radius of Sun
    udtSun.ElongationGeo = 0
    udtSun.ElongationTopo = 0
End Sub
Friend Sub ConvCoords(udtBody As HBody, dblFix As Double, udtWeatherNow As WeatherNow)
    'From J2000 date, ecliptic coordinates and distance, calculate the other coordinates
    Dim dblM As Double
    Dim dblC As Double
    Dim dblNu As Double
    Dim dblOmega As Double
    Dim dblEps As Double
    Dim dblT As Double
    Dim dblLatGeo As Double
    Dim dblRho As Double
    Dim dblSinPar As Double
    Dim dblRefr As Double
    Dim cLST As New clsLST
    Dim dblTheta As Double
    
    dblT = ToJ2000(dblFix)
    'Evaluate nutation parameters
    dblM = FracCirc(357.52911 + dblT * (35999.05029 - 0.0001537 * dblT))
    dblC = (1.914602 - dblT * (0.004817 + 0.000014 * dblT)) * sinD(dblM) + (0.019993 - 0.000101 * dblT) * sinD(2 * dblM) + 0.000289 * sinD(3 * dblM)
    dblNu = dblM + dblC
    dblOmega = 125.04 - 1934.136 * dblT 'Used for nutation correction
    dblEps = Eps(dblT) + 0.00256 * cosD(dblOmega)
    'Convert to RA and declination
    udtBody.RAGeo = atn2D(cosD(dblEps) * sinD(udtBody.EclLong) - tanD(udtBody.EclDecl) * sinD(dblEps), cosD(udtBody.EclLong))
    udtBody.DeclGeo = asinD(sinD(udtBody.EclDecl) * cosD(dblEps) + cosD(udtBody.EclDecl) * sinD(dblEps) * sinD(udtBody.EclLong))
    'Use LST to find hour angle
    udtBody.HourAngGeo = cLST.FixToLST(dblFix, False) - udtBody.RAGeo
    'Conver to altitude azimuth
    udtBody.AzGeo = FracCirc(atn2D(sinD(udtBody.HourAngGeo), cosD(udtBody.HourAngGeo) * sinD(udtLocation.dblLatitude) - tanD(udtBody.DeclGeo) * cosD(udtLocation.dblLatitude)) + 180)
    udtBody.AltGeo = asinD(sinD(udtLocation.dblLatitude) * sinD(udtBody.DeclGeo) + cosD(udtLocation.dblLatitude) * cosD(udtBody.DeclGeo) * cosD(udtBody.HourAngGeo))
    'Correct for parallax
    dblLatGeo = atn2D(Square(0.99664719) * tanD(udtLocation.dblLatitude), 1)
    dblRho = (6378.14 + udtLocation.dblHeight / 1000) / 6378.14
    dblSinPar = sinD(365459.06 / udtBody.DistanceGeo)
    udtBody.HourAngTopo = atn2D(dblRho * cosD(dblLatGeo) * dblSinPar * sinD(udtBody.HourAngGeo), cosD(udtBody.DeclGeo) - dblRho * cosD(dblLatGeo) * dblSinPar * cosD(udtBody.HourAngGeo))
    If udtBody.HourAngTopo > 180 Then udtBody.HourAngTopo = udtBody.HourAngTopo - 360
    udtBody.DeclTopo = atn2D((sinD(udtBody.DeclGeo) - dblRho * sinD(dblLatGeo) * dblSinPar) * cosD(udtBody.HourAngTopo), cosD(udtBody.DeclGeo) - dblRho * cosD(dblLatGeo) * dblSinPar * cosD(udtBody.HourAngGeo))
    If udtBody.DeclTopo > 180 Then udtBody.DeclTopo = udtBody.DeclTopo - 360
    udtBody.RATopo = udtBody.RAGeo - udtBody.HourAngTopo
    udtBody.HourAngTopo = udtBody.HourAngGeo + udtBody.HourAngTopo
    udtBody.AzTopo = FracCirc(atn2D(sinD(udtBody.HourAngTopo), cosD(udtBody.HourAngTopo) * sinD(udtLocation.dblLatitude) - tanD(udtBody.DeclTopo) * cosD(udtLocation.dblLatitude)) + 180)
    udtBody.AltTopo = asinD(sinD(udtLocation.dblLatitude) * sinD(udtBody.DeclTopo) + cosD(udtLocation.dblLatitude) * cosD(udtBody.DeclTopo) * cosD(udtBody.HourAngTopo))
    dblTheta = sinD(dblLatGeo) * sinD(udtBody.DeclTopo) + cosD(dblLatGeo) * cosD(udtBody.DeclTopo) * cosD(udtBody.HourAngGeo)
    If dblTheta > 0.999995 Then
        dblTheta = Sqr(Square(udtBody.HourAngGeo) * cosD((dblLatGeo + udtBody.DeclTopo) / 2) + Square(dblLatGeo - udtBody.DeclTopo))
    ElseIf dblTheta <= -1 Then
        dblTheta = 180
    Else
        dblTheta = acosD(dblTheta)
    End If
    udtBody.DistanceTopo = Sqr(Square(udtBody.DistanceGeo) + Square(dblRho * 6378.14) - 2 * udtBody.DistanceGeo * dblRho * 6378.14 * cosD(dblTheta))
    'Correct for refraction
    dblRefr = Refract(udtBody.AltTopo, True, udtWeatherNow)
    If dblRefr = 90 Then
        udtBody.AltRefr = -90 'Too far below the horizon to calculate
    Else
        udtBody.AltRefr = udtBody.AltTopo + Refract(udtBody.AltTopo, True, udtWeatherNow)
    End If
End Sub
Friend Sub RiseSet(lngFix As Long, blnSet As Boolean, dblRiseSetTime As Double, blnApparent As Boolean, blnNeverRises As Boolean, blnNeverSets As Boolean, Optional dblAltitude As Double = 0, Optional blnSeaLevelHorizon As Boolean = False)
'Find Sunrise or Sunset for a particular day (lngFix as fixed radix)
    Dim cLST As New clsLST
    Dim dblTheta0 As Double
    Dim udtSun(3) As HBody
    Dim dblH0 As Double
    Dim dblH As Double
    Dim dblHH As Double
    Dim dblY(3) As Double
    Dim dblAlpha As Double
    Dim dblDelta As Double
    Dim cDeltaT As New clsDeltaT
    Dim dblDip As Double
    Dim udtWeatherNow As WeatherNow
    Dim dblMidday As Double
    
    Call Meridian(lngFix, True, dblMidday)
    'Find celestial longitude at meridian at mean solar midday
    dblTheta0 = cLST.FixToLST(lngFix + 0.5 - udtLocation.dblLongitude / 360, False)
    'Find the position of the Sun at mean solar midday and for the days before and after
    Call TrueSun(lngFix - 0.5 - udtLocation.dblLongitude / 360, udtSun(0))
    Call TrueSun(lngFix + 0.5 - udtLocation.dblLongitude / 360, udtSun(1))
    Call TrueSun(lngFix + 1.5 - udtLocation.dblLongitude / 360, udtSun(2))
    'Determine the average weather
    Call frmWeather.GetWeatherNow(lngFix + 0.5, 0, udtWeatherNow)
    'Find the altitude at Sunset
    If blnApparent Then
        dblDip = -udtSun(1).Radius - Refract(0, False, udtWeatherNow) + dblAltitude
        If blnSeaLevelHorizon Then dblDip = dblDip - SeaLevelHorizon(udtWeatherNow)
    Else
        dblDip = dblAltitude
    End If
    'The hour angle
    dblH0 = sinD(dblDip) - sinD(udtLocation.dblLatitude) * sinD(udtSun(1).DeclTopo) / cosD(udtLocation.dblLatitude) / cosD(udtSun(1).DeclTopo)
    'If there is no rising or setting then determine midnight or midday
    blnNeverRises = dblH0 > 1
    blnNeverSets = dblH0 < -1
    If dblH0 < -1 Then
        dblH0 = -1
    ElseIf dblH0 > 1 Then
        dblH0 = 1
    End If
    'First approximation
    If blnSet Then
        dblRiseSetTime = (udtSun(1).RATopo - dblTheta0 + acosD(dblH0) - udtLocation.dblLongitude) / 360 + 0.5
        While dblRiseSetTime < dblMidday - 0.25 - lngFix
            dblRiseSetTime = dblRiseSetTime + 1
        Wend
        While dblRiseSetTime >= dblMidday + 0.75 - lngFix
            dblRiseSetTime = dblRiseSetTime - 1
        Wend
    Else
        dblRiseSetTime = FracCirc(udtSun(1).RATopo - dblTheta0 - acosD(dblH0) - udtLocation.dblLongitude) / 360 + 0.5
        While dblRiseSetTime < dblMidday - 0.75 - lngFix
            dblRiseSetTime = dblRiseSetTime + 1
        Wend
        While dblRiseSetTime >= dblMidday + 0.25 - lngFix
            dblRiseSetTime = dblRiseSetTime - 1
        Wend
    End If
    dblY(0) = udtSun(0).RATopo
    dblY(1) = udtSun(1).RATopo
    dblY(2) = udtSun(2).RATopo
    dblAlpha = Interp(dblY, dblRiseSetTime + udtLocation.dblLongitude / 360 + 0.5, 3)
    dblY(0) = udtSun(0).DeclTopo
    dblY(1) = udtSun(1).DeclTopo
    dblY(2) = udtSun(2).DeclTopo
    dblDelta = Interp(dblY, dblRiseSetTime + udtLocation.dblLongitude / 360 + 0.5, 3)
    If Abs(dblH0) > 0.98 Then 'May exceed 1 after iteration so result may be meaningless
        Call frmWeather.GetWeatherNow(lngFix + 0.5, 0, udtWeatherNow)
        Call TrueSun(lngFix + dblRiseSetTime, udtSun(1))
        If blnApparent Then
            dblDip = -udtSun(1).Radius - Refract(0, False, udtWeatherNow) + dblAltitude
            If blnSeaLevelHorizon Then dblDip = dblDip - SeaLevelHorizon(udtWeatherNow)
        Else
            dblDip = dblAltitude
        End If
        dblH0 = sinD(dblDip) - sinD(udtLocation.dblLatitude) * sinD(dblDelta) / cosD(udtLocation.dblLatitude) / cosD(dblDelta)
        blnNeverRises = dblH0 > 1
        blnNeverSets = dblH0 < -1
        If dblH0 < -1 Then
            dblH0 = -1
        ElseIf dblH0 > 1 Then
            dblH0 = 1
        End If
        If blnSet Then
            dblRiseSetTime = (dblAlpha - dblTheta0 + acosD(dblH0)) / 360
            While dblRiseSetTime < dblMidday - 0.25 - lngFix
                dblRiseSetTime = dblRiseSetTime + 1
            Wend
            While dblRiseSetTime >= dblMidday + 0.75 - lngFix
                dblRiseSetTime = dblRiseSetTime - 1
            Wend
        Else
            dblRiseSetTime = FracCirc(dblAlpha - dblTheta0 - acosD(dblH0)) / 360
            While dblRiseSetTime < dblMidday - 0.75 - lngFix
                dblRiseSetTime = dblRiseSetTime + 1
            Wend
            While dblRiseSetTime >= dblMidday + 0.25 - lngFix
                dblRiseSetTime = dblRiseSetTime - 1
            Wend
        End If
    Else
'    Call TrueSun(Int(dblFix) + dblRiseSetTime, udtSun(0))
'    Debug.Print -dblDip, udtSun(0).AltTopo
        dblH = cLST.FixToLST(lngFix + dblRiseSetTime, False)
        dblHH = asinD(sinD(udtLocation.dblLatitude) * sinD(dblDelta) + cosD(udtLocation.dblLatitude) * cosD(dblDelta) * cosD(dblH - dblAlpha))
        dblRiseSetTime = dblRiseSetTime + (dblHH - dblDip) / 360 / cosD(dblDelta) / cosD(udtLocation.dblLatitude) / sinD(dblH - dblAlpha)
    End If
    dblRiseSetTime = dblRiseSetTime + lngFix
'    Call TrueSun(Int(dblFix) + dblRiseSetTime, udtSun(0))
'    Debug.Print -dblDip, udtSun(0).AltTopo
End Sub
Friend Sub Meridian(lngFix As Long, blnNoon As Boolean, dblMeridianTime As Double)
'Find Noon or Midnight for a particular day (lngFix as fixed radix)
    Dim cLST As New clsLST
    Dim dblTheta0 As Double
    Dim udtSun(3) As HBody
    Dim dblH0 As Double
    Dim dblH As Double
    Dim dblHH As Double
    Dim dblY(3) As Double
    Dim dblAlpha As Double
    Dim dblDelta As Double
    Dim cDeltaT As New clsDeltaT
    Dim dblDip As Double
    Dim udtWeatherNow As WeatherNow
    
    'Find celestial longitude at meridian at mean solar midday
    dblTheta0 = cLST.FixToLST(lngFix + 0.5 - udtLocation.dblLongitude / 360, False)
    'Find the position of the Sun at mean solar midday and for the days before and after
    Call TrueSun(lngFix - 0.5 - udtLocation.dblLongitude / 360, udtSun(0))
    Call TrueSun(lngFix + 0.5 - udtLocation.dblLongitude / 360, udtSun(1))
    Call TrueSun(lngFix + 1.5 - udtLocation.dblLongitude / 360, udtSun(2))
    'First approximation
    If blnNoon Then
        dblMeridianTime = (udtSun(1).RATopo - dblTheta0 - udtLocation.dblLongitude) / 360 + 0.5
        'Ensure that Noon is between midnight mean solar time and the next midnight
        While dblMeridianTime < -udtLocation.dblLongitude / 360
            dblMeridianTime = dblMeridianTime + 1
        Wend
        While dblMeridianTime >= 1 - udtLocation.dblLongitude / 360
            dblMeridianTime = dblMeridianTime - 1
        Wend
    Else
        dblMeridianTime = FracCirc(udtSun(1).RATopo - dblTheta0 - udtLocation.dblLongitude) / 360
        'Ensure that Midnight is between the previous noon mean solar time and this noon
        While dblMeridianTime < -0.5 - udtLocation.dblLongitude / 360
            dblMeridianTime = dblMeridianTime + 1
        Wend
        While dblMeridianTime >= 0.5 - udtLocation.dblLongitude / 360
            dblMeridianTime = dblMeridianTime - 1
        Wend
    End If
    'Interpolate to increase accuracy
    dblY(0) = udtSun(0).RATopo
    dblY(1) = udtSun(1).RATopo
    dblY(2) = udtSun(2).RATopo
    dblAlpha = Interp(dblY, dblMeridianTime + udtLocation.dblLongitude / 360 + 0.5, 3)
    dblY(0) = udtSun(0).DeclTopo
    dblY(1) = udtSun(1).DeclTopo
    dblY(2) = udtSun(2).DeclTopo
    dblDelta = Interp(dblY, dblMeridianTime + udtLocation.dblLongitude / 360 + 0.5, 3)
    dblH = cLST.FixToLST(lngFix + dblMeridianTime, False)
    If Not blnNoon Then dblH = dblH - 180
    dblMeridianTime = dblMeridianTime - (dblH - dblAlpha) / 360 + lngFix
End Sub
Friend Function ToJ2000(dblFix As Double) As Double
'Julian centuries since J2000
    Dim cJDE As New clsJDE
    Dim udtDateOut As MyDate
    Dim udtTimeOut As Mytime
    
    ToJ2000 = (cJDE.fixed_jde(dblFix) - 2451545) / 36525
End Function
Friend Function Refract(dblAltitude As Double, blnToApparent As Boolean, udtWeatherNow As WeatherNow) As Double
'Calculate the adjustment for refraction
    If blnToApparent Then
        If dblAltitude > 0.25 Then
            'Formula from Astronomical Algorithms
            Refract = 1.02 / tanD(dblAltitude + 10.3 / (dblAltitude + 5.11)) / 60 * 273 / 283
        ElseIf dblAltitude > -0.3 Then
            'Mixture of last and next formula for continuity
            Refract = dblAltitude * (0.033331 + dblAltitude * 0.030716)
            Refract = 0.60507 - dblAltitude * (0.30781 - dblAltitude * (0.064184 + Refract))
            Refract = Refract * (0.3 - dblAltitude) / 0.6 + 1.02 / tanD(dblAltitude + 10.3 / (dblAltitude + 5.11)) / 60 * 273 / 283 * (0.3 + dblAltitude) / 0.6
        ElseIf dblAltitude > -SeaLevelHorizon(udtWeatherNow) - 1.608 And dblAltitude > -4 Then 'Lowest possible horizon of 450 m below seal level
            'Quartic fit to Garstang 1969
            Refract = dblAltitude * (0.033331 + dblAltitude * 0.030716)
            Refract = 0.60507 - dblAltitude * (0.30781 - dblAltitude * (0.064184 + Refract))
        Else
            Refract = -90 'Not within range
        End If
    Else
        If dblAltitude > -0.060507 Then
            Refract = 1 / tanD(dblAltitude + 7.31 / (dblAltitude + 4.4)) / 60 * 273 / 283
        ElseIf dblAltitude > SeaLevelHorizon(udtWeatherNow) And dblAltitude > -4 Then
            Refract = 3.7789 - dblAltitude * (9.5427 - dblAltitude * (4.8728 - dblAltitude * (1.3112 - dblAltitude * 0.13107)))
        Else
            Refract = 90
        End If
    End If
    If Refract <> 90 Then Refract = Refract * udtWeatherNow.dblPressure / 1010 * 273 / (273 + udtWeatherNow.dblTemperature)
End Function
Friend Function EclipticCoordinates() As String
'Return coordinates as string
    Dim udtSun As HBody
    Dim udtDms As DMS
    Dim cDMS As New clsDMS
    
    Call TrueSun(dblDate + dblTime, udtSun)
    Call cDMS.ToDMS(udtSun.EclLong, udtDms)
    EclipticCoordinates = "Sun's geocentric ecliptic coordinates" & vbCrLf & "Longitude " & udtDms.intDegreesHours & "" & udtDms.intMinutes & "'" & Int(udtDms.dblSeconds) & """" & vbCrLf
    Call cDMS.ToDMS(udtSun.EclDecl, udtDms)
    EclipticCoordinates = EclipticCoordinates & "Declination "
    If Not udtDms.blnSign Then EclipticCoordinates = EclipticCoordinates & "-"
    EclipticCoordinates = EclipticCoordinates & udtDms.intDegreesHours & "" & udtDms.intMinutes & "'" & Int(udtDms.dblSeconds) & """"
End Function
Friend Function EquatorialCoordinates() As String
'Return coordinates as string
    Dim udtSun As HBody
    Dim udtDms As DMS
    Dim cDMS As New clsDMS
    
    Call TrueSun(dblDate + dblTime, udtSun)
    Call cDMS.ToDMS(udtSun.RAGeo, udtDms)
    EquatorialCoordinates = "Sun's geocentric equatorial coordinates" & vbCrLf & "Right ascension " & udtDms.intDegreesHours & "" & udtDms.intMinutes & "'" & Int(udtDms.dblSeconds) & """" & vbCrLf
    Call cDMS.ToDMS(udtSun.RAGeo / 15, udtDms)
    EquatorialCoordinates = EquatorialCoordinates & "              = " & udtDms.intDegreesHours & "h" & udtDms.intMinutes & "m" & Int(udtDms.dblSeconds) & "s" & vbCrLf
    Call cDMS.ToDMS(udtSun.DeclGeo, udtDms)
    EquatorialCoordinates = EquatorialCoordinates & "Declination "
    If Not udtDms.blnSign Then EquatorialCoordinates = EquatorialCoordinates & "-"
    EquatorialCoordinates = EquatorialCoordinates & udtDms.intDegreesHours & "" & udtDms.intMinutes & "'" & Int(udtDms.dblSeconds) & """" & vbCrLf
    Call cDMS.ToDMS(udtSun.RATopo, udtDms)
    EquatorialCoordinates = EquatorialCoordinates & "Sun's topocentric equatorial coordinates" & vbCrLf & "Right ascension " & udtDms.intDegreesHours & "" & udtDms.intMinutes & "'" & Int(udtDms.dblSeconds) & """" & vbCrLf
    Call cDMS.ToDMS(udtSun.RATopo / 15, udtDms)
    EquatorialCoordinates = EquatorialCoordinates & "              = " & udtDms.intDegreesHours & "h" & udtDms.intMinutes & "m" & Int(udtDms.dblSeconds) & "s" & vbCrLf
    Call cDMS.ToDMS(udtSun.DeclTopo, udtDms)
    EquatorialCoordinates = EquatorialCoordinates & "Declination "
    If Not udtDms.blnSign Then EquatorialCoordinates = EquatorialCoordinates & "-"
    EquatorialCoordinates = EquatorialCoordinates & udtDms.intDegreesHours & "" & udtDms.intMinutes & "'" & Int(udtDms.dblSeconds) & """"
End Function
Friend Function HorizonCoordinates() As String
'Return coordinates as string
    Dim udtSun As HBody
    Dim udtDms As DMS
    Dim cDMS As New clsDMS
    
    Call TrueSun(dblDate + dblTime, udtSun)
    Call cDMS.ToDMS(udtSun.AzGeo, udtDms)
    HorizonCoordinates = "Sun's geocentric horizon coordinates" & vbCrLf & "Azimuth " & udtDms.intDegreesHours & "" & udtDms.intMinutes & "'" & Int(udtDms.dblSeconds) & """" & vbCrLf
    Call cDMS.ToDMS(udtSun.AltGeo, udtDms)
    HorizonCoordinates = HorizonCoordinates & "Altitude "
    If Not udtDms.blnSign Then HorizonCoordinates = HorizonCoordinates & "-"
    HorizonCoordinates = HorizonCoordinates & udtDms.intDegreesHours & "" & udtDms.intMinutes & "'" & Int(udtDms.dblSeconds) & """" & vbCrLf
    Call cDMS.ToDMS(udtSun.AzTopo, udtDms)
    HorizonCoordinates = HorizonCoordinates & "Sun's topocentric horizon coordinates" & vbCrLf & "Azimuth " & udtDms.intDegreesHours & "" & udtDms.intMinutes & "'" & Int(udtDms.dblSeconds) & """" & vbCrLf
    Call cDMS.ToDMS(udtSun.AltTopo, udtDms)
    HorizonCoordinates = HorizonCoordinates & "Altitude "
    If Not udtDms.blnSign Then HorizonCoordinates = HorizonCoordinates & "-"
    HorizonCoordinates = HorizonCoordinates & udtDms.intDegreesHours & "" & udtDms.intMinutes & "'" & Int(udtDms.dblSeconds) & """" & vbCrLf
    If udtSun.AltRefr <> -90 Then
        Call cDMS.ToDMS(udtSun.AltRefr, udtDms)
        HorizonCoordinates = HorizonCoordinates & "After correction for refraction" & vbCrLf & "Altitude "
        If Not udtDms.blnSign Then HorizonCoordinates = HorizonCoordinates & "-"
        HorizonCoordinates = HorizonCoordinates & udtDms.intDegreesHours & "" & udtDms.intMinutes & "'" & Int(udtDms.dblSeconds) & """"
    Else
        HorizonCoordinates = HorizonCoordinates & "The Sun is well below the horizon so refraction cannot be calculated."
    End If
End Function
Friend Function DistSize() As String
'Return distance and apparent radius
    Dim udtSun As HBody
    Dim udtDms As DMS
    Dim cDMS As New clsDMS
    
    Call TrueSun(dblDate + dblTime, udtSun)
    DistSize = "Sun's distance and size" & vbCrLf & "Geocentric distance " & Format(udtSun.DistanceGeo, "000000000") & " km" & vbCrLf
    DistSize = DistSize & "Topocentric distance " & Format(udtSun.DistanceTopo, "000000000") & " km" & vbCrLf
    Call cDMS.ToDMS(udtSun.Radius, udtDms)
    DistSize = DistSize & "Topocentric radius " & udtDms.intMinutes & "'" & Int(udtDms.dblSeconds) & """"
End Function
Friend Function Sunrise() As String
'Return sunrise for that day
    Dim dblRiseSetTime As Double
    Dim blnNeverSets As Boolean
    Dim blnNeverRises As Boolean
    Dim intDays As Integer
    Dim udtDms As DMS
    Dim lngFix As Long
    Dim cDMS As New clsDMS
    Dim cMoon As New clsMoon
    Dim udtDateOut As MyDate
    Dim udtTimeOut As Mytime
    Dim blnValid As Boolean
    Dim strOldDate As String
    Dim strDate As String
    
    strOldDate = ""
    blnValid = True
    lngFix = frmDate.ToLngDate(dblDate + dblTime)
    Call RiseSet(lngFix, False, dblRiseSetTime, False, blnNeverRises, blnNeverSets)
    If blnNeverSets Then
        Sunrise = "No geometric sunset" & vbCrLf & "Minimum altitude at "
    ElseIf blnNeverRises Then
        Sunrise = "No geometric sunrise" & vbCrLf & "Maximum altitude at "
    Else
        Sunrise = "Geometric Sunrise "
    End If
    Call frmMain.ToDateTime(dblRiseSetTime, udtDateOut, udtDate.enmDateType, udtTimeOut, udtTime.enmTimeType, blnValid)
    Sunrise = Sunrise & frmDate.WriteDateTime(udtDateOut, udtTimeOut, strOldDate)
    If Not blnNeverSets Then
        Call RiseSet(lngFix, False, dblRiseSetTime, True, blnNeverRises, blnNeverSets)
        If blnNeverSets Then
            Sunrise = Sunrise & vbCrLf & "No apparent sunset over flat horizon" & vbCrLf & "Minimum altitude at "
        ElseIf blnNeverRises Then
            Sunrise = Sunrise & vbCrLf & "No apparent sunrise over flat horizon" & vbCrLf & "Maximum altitude at "
        Else
            Sunrise = Sunrise & vbCrLf & "Apparent Sunrise over flat horizon "
        End If
        Call frmMain.ToDateTime(dblRiseSetTime, udtDateOut, udtDate.enmDateType, udtTimeOut, udtTime.enmTimeType, blnValid)
        Sunrise = Sunrise & frmDate.WriteDateTime(udtDateOut, udtTimeOut, strOldDate)
        If Not blnNeverSets Then
            Call RiseSet(lngFix, False, dblRiseSetTime, True, blnNeverRises, blnNeverSets, 0, True)
            If blnNeverSets Then
                Sunrise = Sunrise & vbCrLf & "No apparent sunset over sea level horizon" & vbCrLf & "Minimum altitude at "
            ElseIf blnNeverRises Then
                Sunrise = Sunrise & vbCrLf & "No apparent sunrise over sea level horizon" & vbCrLf & "Maximum altitude at "
            Else
                Sunrise = Sunrise & vbCrLf & "Apparent Sunrise over sea level horizon "
            End If
            Call frmMain.ToDateTime(dblRiseSetTime, udtDateOut, udtDate.enmDateType, udtTimeOut, udtTime.enmTimeType, blnValid)
            Sunrise = Sunrise & frmDate.WriteDateTime(udtDateOut, udtTimeOut, strOldDate)
        End If
    End If
End Function
Friend Function Sunset() As String
'Return Sunset for that day
    Dim dblRiseSetTime As Double
    Dim blnNeverSets As Boolean
    Dim blnNeverRises As Boolean
    Dim intDays As Integer
    Dim udtDms As DMS
    Dim lngFix As Long
    Dim cDMS As New clsDMS
    Dim cMoon As New clsMoon
    Dim udtDateOut As MyDate
    Dim udtTimeOut As Mytime
    Dim blnValid As Boolean
    Dim strOldDate As String
    
    strOldDate = ""
    blnValid = True
    lngFix = frmDate.ToLngDate(dblDate + dblTime)
    Call RiseSet(lngFix, True, dblRiseSetTime, False, blnNeverRises, blnNeverSets)
    If blnNeverSets Then
        Sunset = "No geometric sunset" & vbCrLf & "Minimum altitude at "
    ElseIf blnNeverRises Then
        Sunset = "No geometric sunrise" & vbCrLf & "Maximum altitude at "
    Else
        Sunset = "Geometric Sunset "
    End If
    Call frmMain.ToDateTime(dblRiseSetTime, udtDateOut, udtDate.enmDateType, udtTimeOut, udtTime.enmTimeType, blnValid)
    Sunset = Sunset & frmDate.WriteDateTime(udtDateOut, udtTimeOut, strOldDate)
    If Not blnNeverSets Then
        Call RiseSet(lngFix, True, dblRiseSetTime, True, blnNeverRises, blnNeverSets)
        If blnNeverSets Then
            Sunset = Sunset & vbCrLf & "No apparent sunset over flat horizon" & vbCrLf & "Minimum altitude at "
        ElseIf blnNeverRises Then
            Sunset = Sunset & vbCrLf & "No apparent sunrise over flat horizon" & vbCrLf & "Maximum altitude at "
        Else
            Sunset = Sunset & vbCrLf & "Apparent Sunset over flat horizon"
        End If
        Call frmMain.ToDateTime(dblRiseSetTime, udtDateOut, udtDate.enmDateType, udtTimeOut, udtTime.enmTimeType, blnValid)
        Sunset = Sunset & frmDate.WriteDateTime(udtDateOut, udtTimeOut, strOldDate)
        If Not blnNeverSets And udtLocation.dblHeight >= 0 Then
        Call RiseSet(lngFix, True, dblRiseSetTime, True, blnNeverRises, blnNeverSets, 0, True)
            If blnNeverSets Then
                Sunset = Sunset & vbCrLf & "No apparent sunset over sea level horizon" & vbCrLf & "Minimum altitude at "
            ElseIf blnNeverRises Then
                Sunset = Sunset & vbCrLf & "No apparent sunrise over sea level horizon" & vbCrLf & "Maximum altitude at "
            Else
                Sunset = Sunset & vbCrLf & "Apparent Sunset over sea level horizon"
            End If
            Call frmMain.ToDateTime(dblRiseSetTime, udtDateOut, udtDate.enmDateType, udtTimeOut, udtTime.enmTimeType, blnValid)
            Sunset = Sunset & frmDate.WriteDateTime(udtDateOut, udtTimeOut, strOldDate)
        End If
    End If
End Function
Friend Function SeaLevelHorizon(udtWeartherNow As WeatherNow) As Double
'Calculate apparent angle of dip to the sea
    Dim dblPSL As Double
    Dim dblTSL As Double
    
    If udtLocation.dblHeight > 0 Then
        dblPSL = udtWeatherNow.dblPressure * Exp(udtLocation.dblHeight / 7143) / 1013.25
        dblTSL = (udtWeatherNow.dblTemperature + 0.005688 * udtLocation.dblHeight + 273.15) / 273.15
        SeaLevelHorizon = (0.24 - 0.23737 * Exp(-0.016594 * Sqr(udtLocation.dblHeight))) * dblPSL / dblTSL + acosD(6378140 / (udtLocation.dblHeight + 6378140))
    Else
        SeaLevelHorizon = 0
    End If
End Function
Friend Function Magnitude(udtSun As HBody) As Double
'Calculate the magnitude of the Sun as seen from above the atmosphere
    Magnitude = -27.3 + 5 * Log10(udtSun.DistanceTopo / AU)
End Function
Friend Function MagText() As String
    Dim udtSun As HBody
    
    Call TrueSun(dblDate + dblTime, udtSun)
    MagText = "Sun's magnitude above the atmosphere " & Format(Magnitude(udtSun), "###.00")
End Function

